#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import SCHOOL, WORKDAY
from holidays.countries.yemen import Yemen
from tests.common import CommonCountryTests


class TestYemen(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Yemen)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(Yemen(categories=SCHOOL, years=range(self.start_year, 2013)))
        self.assertNoHolidays(Yemen(categories=WORKDAY, years=range(self.start_year, 2000)))

    def test_labor_day(self):
        name = "عيد العمال"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2009-05-02",
            "2015-05-03",
            "2020-05-03",
        )
        self.assertHolidayName(f"{name} (يوم تعويضي)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_unity_day(self):
        name = "اليوم الوطني للجمهورية اليمنية"
        self.assertHolidayName(name, (f"{year}-05-22" for year in self.full_range))
        obs_dts = (
            "2009-05-23",
            "2015-05-24",
            "2020-05-27",
        )
        self.assertHolidayName(f"{name} (يوم تعويضي)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_revolution_day(self):
        name = "ثورة 26 سبتمبر المجيدة"
        self.assertHolidayName(name, (f"{year}-09-26" for year in self.full_range))
        obs_dts = (
            "2003-09-27",
            "2008-09-27",
            "2014-09-28",
        )
        self.assertHolidayName(f"{name} (يوم تعويضي)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_liberation_day(self):
        name = "ثورة 14 أكتوبر المجيدة"
        self.assertHolidayName(name, (f"{year}-10-14" for year in self.full_range))
        obs_dts = (
            "2011-10-15",
            "2016-10-16",
            "2022-10-16",
        )
        self.assertHolidayName(f"{name} (يوم تعويضي)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        name = "عيد الجلاء"
        self.assertHolidayName(name, (f"{year}-11-30" for year in self.full_range))
        obs_dts = (
            "2007-12-01",
            "2012-12-01",
            "2018-12-02",
        )
        self.assertHolidayName(f"{name} (يوم تعويضي)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_hijri_new_year(self):
        name = "عيد رأس السنة الهجرية"
        self.assertHolidayName(
            name,
            "2020-08-20",
            "2021-08-10",
            "2022-07-30",
            "2023-07-19",
            "2024-07-07",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "1994-06-11",
            "2002-03-16",
            "2009-12-19",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (يوم تعويضي)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_fitr(self):
        name = "عيد الفطر"
        self.assertHolidayName(
            name,
            "2020-05-23",
            "2020-05-24",
            "2020-05-25",
            "2020-05-26",
            "2021-05-12",
            "2021-05-13",
            "2021-05-14",
            "2021-05-15",
            "2022-05-01",
            "2022-05-02",
            "2022-05-03",
            "2022-05-04",
            "2023-04-20",
            "2023-04-21",
            "2023-04-22",
            "2023-04-23",
            "2024-04-08",
            "2024-04-09",
            "2024-04-10",
            "2024-04-11",
            "2024-04-12",
            "2025-03-29",
            "2025-03-30",
            "2025-03-31",
            "2025-04-01",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2020-05-27",
            "2021-05-16",
            "2023-04-24",
            "2024-04-14",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (يوم تعويضي)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_adha(self):
        name = "عيد الأضحى"
        self.assertHolidayName(
            name,
            "2020-07-30",
            "2020-07-31",
            "2020-08-01",
            "2020-08-02",
            "2020-08-03",
            "2021-07-19",
            "2021-07-20",
            "2021-07-21",
            "2021-07-22",
            "2021-07-23",
            "2022-07-08",
            "2022-07-09",
            "2022-07-10",
            "2022-07-11",
            "2022-07-12",
            "2023-06-27",
            "2023-06-28",
            "2023-06-29",
            "2023-06-30",
            "2023-07-01",
            "2024-06-15",
            "2024-06-16",
            "2024-06-17",
            "2024-06-18",
            "2024-06-19",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2020-08-04",
            "2021-07-25",
            "2022-07-13",
            "2023-07-02",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (يوم تعويضي)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_teachers_day(self):
        name = "عيد المعلم"
        self.assertNoHolidayName(name)
        self.assertSchoolHolidayName(
            name, (f"{year}-05-05" for year in range(2013, self.end_year))
        )
        self.assertNoSchoolHolidayName(name, range(self.start_year, 2013))

    def test_mawlid(self):
        name = "المولد النبوي"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "1991-09-20",
            "1992-09-09",
            "1993-08-29",
            "1994-08-19",
            "1995-08-08",
            "1996-07-27",
            "1997-07-16",
            "1998-07-06",
            "1999-06-26",
        )
        self.assertNoIslamicNoEstimatedHolidayName(name, range(2000, self.end_year))
        obs_dts = (
            "1991-09-21",
            "1994-08-20",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (يوم تعويضي)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertWorkdayIslamicNoEstimatedHolidayName(
            name,
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
            "2023-09-27",
            "2024-09-15",
        )
        self.assertWorkdayIslamicNoEstimatedHolidayName(name, range(2000, self.end_year))
        self.assertNoWorkdayIslamicNoEstimatedHolidayName(name, range(self.start_year, 2000))

    def test_isra_and_miraj(self):
        name = "ذكرى الإسراء والمعراج"
        self.assertNoHolidayName(name)
        self.assertWorkdayIslamicNoEstimatedHolidayName(
            name,
            "2023-02-18",
            "2024-02-08",
            "2025-01-27",
        )
        self.assertWorkdayIslamicNoEstimatedHolidayName(name, range(2000, self.end_year))
        self.assertNoWorkdayIslamicNoEstimatedHolidayName(name, range(self.start_year, 2000))

    def test_victory_day(self):
        name = "ذكرى 7 يوليو"
        self.assertHolidayName(name, (f"{year}-07-07" for year in range(self.start_year, 2000)))
        self.assertNoHolidayName(name, range(2000, self.end_year))
        self.assertWorkdayHolidayName(
            name, (f"{year}-07-07" for year in range(2000, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2000))

    def test_weekend(self):
        for dt in (
            "2013-08-08",  # THU.
            "2013-08-09",  # FRI.
            "2013-08-16",  # FRI.
            "2013-08-17",  # SAT.
        ):
            self.assertTrue(self.holidays.is_weekend(dt))

        for dt in (
            "2013-08-10",  # SAT.
            "2013-08-11",  # SUN.
            "2013-08-15",  # THU.
            "2013-08-18",  # SUN.
        ):
            self.assertFalse(self.holidays.is_weekend(dt))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-02-18", "ذكرى الإسراء والمعراج"),
            ("2023-04-20", "عيد الفطر"),
            ("2023-04-21", "عيد الفطر"),
            ("2023-04-22", "عيد الفطر"),
            ("2023-04-23", "عيد الفطر"),
            ("2023-04-24", "عيد الفطر (يوم تعويضي)"),
            ("2023-05-01", "عيد العمال"),
            ("2023-05-05", "عيد المعلم"),
            ("2023-05-22", "اليوم الوطني للجمهورية اليمنية"),
            ("2023-06-27", "عيد الأضحى"),
            ("2023-06-28", "عيد الأضحى"),
            ("2023-06-29", "عيد الأضحى"),
            ("2023-06-30", "عيد الأضحى"),
            ("2023-07-01", "عيد الأضحى"),
            ("2023-07-02", "عيد الأضحى (يوم تعويضي)"),
            ("2023-07-07", "ذكرى 7 يوليو"),
            ("2023-07-19", "عيد رأس السنة الهجرية"),
            ("2023-09-26", "ثورة 26 سبتمبر المجيدة"),
            ("2023-09-27", "المولد النبوي"),
            ("2023-10-14", "ثورة 14 أكتوبر المجيدة"),
            ("2023-11-30", "عيد الجلاء"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-02-18", "Isra' and Mi'raj"),
            ("2023-04-20", "Eid al-Fitr"),
            ("2023-04-21", "Eid al-Fitr"),
            ("2023-04-22", "Eid al-Fitr"),
            ("2023-04-23", "Eid al-Fitr"),
            ("2023-04-24", "Eid al-Fitr (observed)"),
            ("2023-05-01", "Labor Day"),
            ("2023-05-05", "Teacher's Day"),
            ("2023-05-22", "Unity Day"),
            ("2023-06-27", "Eid al-Adha"),
            ("2023-06-28", "Eid al-Adha"),
            ("2023-06-29", "Eid al-Adha"),
            ("2023-06-30", "Eid al-Adha"),
            ("2023-07-01", "Eid al-Adha"),
            ("2023-07-02", "Eid al-Adha (observed)"),
            ("2023-07-07", "Victory Day"),
            ("2023-07-19", "Hijri New Year"),
            ("2023-09-26", "Revolution Day"),
            ("2023-09-27", "Prophet's Birthday"),
            ("2023-10-14", "Liberation Day"),
            ("2023-11-30", "Independence Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import GREGORIAN_CALENDAR, MAR, OCT, FRI, SAT
from holidays.calendars.julian import JULIAN_CALENDAR
from holidays.groups import ChristianHolidays, InternationalHolidays, IslamicHolidays
from holidays.holiday_base import HolidayBase


class SyrianArabRepublic(HolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays):
    """Syrian Arab Republic holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Syria>
        * <https://en.wikipedia.org/wiki/Liberation_Day_(Syria)>
        * [Central Bank of Syria - Public Holidays 2024](https://web.archive.org/web/20250813035705/https://cb.gov.sy/index.php?lang=1&dir=html&ex=1&page=holidays&act=598-)
        * [Christmas and Boxing Day](https://web.archive.org/web/20250414224148/https://en.royanews.tv/news/56308)
    """

    country = "SY"
    default_language = "ar"
    # %s (estimated).
    estimated_label = tr("%s (تقديري)")
    # Earliest reliable record dates back to 2004.
    start_year = 2004
    supported_languages = ("ar", "en_US")
    weekend = {FRI, SAT}

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self, calendar=JULIAN_CALENDAR)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=SyriaIslamicHolidays, show_estimated=islamic_show_estimated
        )
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("عيد رأس السنة الميلادية"))

        # Revolution Day.
        self._add_holiday_mar_8(tr("الثورة السورية"))

        # Mother's Day.
        self._add_holiday_mar_21(tr("عيد الأم"))

        # Gregorian Easter Sunday.
        self._add_easter_sunday(tr("عيد الفصح حسب التقويم الغربي"), calendar=GREGORIAN_CALENDAR)

        # Julian Easter Sunday.
        self._add_easter_sunday(tr("عيد الفصح حسب التقويم الشرقي"))

        # Independence Day.
        self._add_holiday_apr_17(tr("عيد الاستقلال"))

        # Labor Day.
        self._add_labor_day(tr("عيد العمال"))

        # Martyrs' Day.
        self._add_holiday_may_6(tr("عيد الشهداء"))

        # Tishreen Liberation War Day.
        self._add_holiday_oct_6(tr("ذكرى حرب تشرين التحريرية"))

        if self._year >= 2025:
            # Liberation Day.
            self._add_holiday_dec_8(tr("عيد التحرير"))

        if self._year >= 2024:
            # Christmas Day.
            self._add_christmas_day(tr("عيد الميلاد"), calendar=GREGORIAN_CALENDAR)

            # Boxing Day.
            self._add_christmas_day_two(tr("يوم الصناديق"), calendar=GREGORIAN_CALENDAR)

        # Islamic New Year.
        self._add_islamic_new_year_day(tr("رأس السنة الهجرية"))

        # Prophet's Birthday.
        self._add_mawlid_day(tr("عيد المولد النبوي الشريف"))

        # Eid al-Fitr.
        name = tr("عيد الفطر")
        self._add_eid_al_fitr_day(name)
        self._add_eid_al_fitr_day_two(name)
        self._add_eid_al_fitr_day_three(name)

        # Eid al-Adha.
        name = tr("عيد الأضحى")
        self._add_arafah_day(name)
        self._add_eid_al_adha_day(name)
        self._add_eid_al_adha_day_two(name)
        self._add_eid_al_adha_day_three(name)


class SY(SyrianArabRepublic):
    pass


class SYR(SyrianArabRepublic):
    pass


class SyriaIslamicHolidays(_CustomIslamicHolidays):
    # https://web.archive.org/web/20250120140034/https://www.timeanddate.com/holidays/syria/muharram-new-year
    HIJRI_NEW_YEAR_DATES_CONFIRMED_YEARS = (2020, 2025)

    # https://web.archive.org/web/20240808150300/https://www.timeanddate.com/holidays/syria/prophet-birthday
    MAWLID_DATES_CONFIRMED_YEARS = (2020, 2024)
    MAWLID_DATES = {
        2021: (OCT, 19),
    }

    # https://web.archive.org/web/20250404011133/https://www.timeanddate.com/holidays/syria/eid-al-fitr
    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2020, 2025)
    EID_AL_FITR_DATES = {
        2025: (MAR, 31),
    }

    # https://web.archive.org/web/20250126132002/https://www.timeanddate.com/holidays/syria/eid-al-adha
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2020, 2025)



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import JAN, MAY, JUL, AUG, SEP, FRI, SAT
from holidays.calendars.julian import JULIAN_CALENDAR
from holidays.groups import ChristianHolidays, IslamicHolidays
from holidays.holiday_base import HolidayBase


class Sudan(HolidayBase, ChristianHolidays, IslamicHolidays):
    """Sudan holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Sudan>
        * <https://web.archive.org/web/20250820070831/https://www.sudanembassy.org.uk/public-holidays/>
        * [Christian Holidays 2011-2018](https://web.archive.org/web/20250827155208/https://evangelicalfocus.com/world/5014/christmas-celebrations-mark-progress-of-religious-freedom-in-sudan)
    """

    country = "SD"
    default_language = "ar_SD"
    # %s (estimated).
    estimated_label = tr("%s (تقديري)")
    supported_languages = ("ar_SD", "en_US")
    start_year = 1985
    weekend = {FRI, SAT}

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        ChristianHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=SudanIslamicHolidays, show_estimated=islamic_show_estimated
        )
        super().__init__(*args, **kwargs)

    def _get_weekend(self, dt: date) -> set[int]:
        # The resting days are Friday and Saturday since January 26th, 2008.
        # https://sudantribune.com/article/25544
        return {FRI, SAT} if dt >= date(2008, JAN, 26) else {FRI}

    def _populate_public_holidays(self):
        # Independence Day.
        self._add_holiday_jan_1(tr("عيد الإستقلال"))

        # Christian public holidays were suspended 2011-2018 and reinstated in 2019.
        if self._year <= 2010 or self._year >= 2019:
            # Coptic Christmas Day.
            self._add_christmas_day(tr("عيد الميلاد المجيد"), calendar=JULIAN_CALENDAR)

            # Coptic Easter.
            self._add_easter_sunday(tr("عيد الفصح القبطي"), calendar=JULIAN_CALENDAR)

            # Christmas Day.
            self._add_christmas_day(tr("عيد الميلاد"))

        # Islamic New Year.
        self._add_islamic_new_year_day(tr("رأس السنة الهجرية"))

        # Prophet's Birthday.
        self._add_mawlid_day(tr("المولد النبوي الشريف"))

        # Eid al-Fitr.
        name = tr("عيد الفطر المبارك")
        self._add_eid_al_fitr_day(name)
        self._add_eid_al_fitr_day_two(name)
        self._add_eid_al_fitr_day_three(name)

        if self._year >= 2020:
            self._add_eid_al_fitr_day_four(name)

        # Eid al-Adha.
        name = tr("عيد الأضحى المبارك")
        self._add_arafah_day(name)
        self._add_eid_al_adha_day(name)
        self._add_eid_al_adha_day_two(name)
        self._add_eid_al_adha_day_three(name)
        self._add_eid_al_adha_day_four(name)


class SD(Sudan):
    pass


class SDN(Sudan):
    pass


class SudanIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2020, 2025)
    EID_AL_ADHA_DATES = {
        2022: (JUL, 10),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2020, 2025)
    EID_AL_FITR_DATES = {
        2022: (MAY, 1),
    }

    HIJRI_NEW_YEAR_DATES_CONFIRMED_YEARS = (2020, 2025)
    HIJRI_NEW_YEAR_DATES = {
        2021: (AUG, 11),
    }

    MAWLID_DATES_CONFIRMED_YEARS = (2020, 2025)
    MAWLID_DATES = {
        2023: (SEP, 28),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.luxembourg import Luxembourg
from tests.common import CommonCountryTests


class TestLuxembourg(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Luxembourg)

    def test_new_years_day(self):
        self.assertHolidayName("Neijoerschdag", (f"{year}-01-01" for year in self.full_range))

    def test_good_friday(self):
        name = "Karfreideg"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertBankHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Ouschterméindeg"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        self.assertHolidayName(
            "Dag vun der Aarbecht", (f"{year}-05-01" for year in self.full_range)
        )

    def test_europe_day(self):
        name = "Europadag"
        self.assertHolidayName(name, (f"{year}-05-09" for year in range(2019, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2019))

    def test_ascension_day(self):
        name = "Christi Himmelfaart"
        self.assertHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_monday(self):
        name = "Péngschtméindeg"
        self.assertHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_national_day(self):
        self.assertHolidayName("Nationalfeierdag", (f"{year}-06-23" for year in self.full_range))

    def test_assumption_day(self):
        self.assertHolidayName("Léiffrawëschdag", (f"{year}-08-15" for year in self.full_range))

    def test_all_saints_day(self):
        self.assertHolidayName("Allerhellgen", (f"{year}-11-01" for year in self.full_range))

    def test_christmas_eve(self):
        name = "Hellegowend (nomëtteg)"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(name, (f"{year}-12-24" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName("Chrëschtdag", (f"{year}-12-25" for year in self.full_range))

    def test_saint_stephens_day(self):
        self.assertHolidayName("Stiefesdag", (f"{year}-12-26" for year in self.full_range))

    def test_new_years_eve(self):
        name = "Silvester"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(name, (f"{year}-12-31" for year in self.full_range))

    def test_2018(self):
        self.assertHolidaysInYear(
            2018,
            ("2018-01-01", "Neijoerschdag"),
            ("2018-04-02", "Ouschterméindeg"),
            ("2018-05-01", "Dag vun der Aarbecht"),
            ("2018-05-10", "Christi Himmelfaart"),
            ("2018-05-21", "Péngschtméindeg"),
            ("2018-06-23", "Nationalfeierdag"),
            ("2018-08-15", "Léiffrawëschdag"),
            ("2018-11-01", "Allerhellgen"),
            ("2018-12-25", "Chrëschtdag"),
            ("2018-12-26", "Stiefesdag"),
        )

    def test_2018_bank(self):
        self.assertBankHolidaysInYear(
            2018,
            ("2018-03-30", "Karfreideg"),
            ("2018-12-24", "Hellegowend (nomëtteg)"),
            ("2018-12-31", "Silvester"),
        )

    def test_2019(self):
        self.assertHolidaysInYear(
            2019,
            ("2019-01-01", "Neijoerschdag"),
            ("2019-04-22", "Ouschterméindeg"),
            ("2019-05-01", "Dag vun der Aarbecht"),
            ("2019-05-09", "Europadag"),
            ("2019-05-30", "Christi Himmelfaart"),
            ("2019-06-10", "Péngschtméindeg"),
            ("2019-06-23", "Nationalfeierdag"),
            ("2019-08-15", "Léiffrawëschdag"),
            ("2019-11-01", "Allerhellgen"),
            ("2019-12-25", "Chrëschtdag"),
            ("2019-12-26", "Stiefesdag"),
        )

    def test_2019_bank(self):
        self.assertBankHolidaysInYear(
            2019,
            ("2019-04-19", "Karfreideg"),
            ("2019-12-24", "Hellegowend (nomëtteg)"),
            ("2019-12-31", "Silvester"),
        )

    def test_2020(self):
        self.assertHolidaysInYear(
            2020,
            ("2020-01-01", "Neijoerschdag"),
            ("2020-04-13", "Ouschterméindeg"),
            ("2020-05-01", "Dag vun der Aarbecht"),
            ("2020-05-09", "Europadag"),
            ("2020-05-21", "Christi Himmelfaart"),
            ("2020-06-01", "Péngschtméindeg"),
            ("2020-06-23", "Nationalfeierdag"),
            ("2020-08-15", "Léiffrawëschdag"),
            ("2020-11-01", "Allerhellgen"),
            ("2020-12-25", "Chrëschtdag"),
            ("2020-12-26", "Stiefesdag"),
        )

    def test_2020_bank(self):
        self.assertBankHolidaysInYear(
            2020,
            ("2020-04-10", "Karfreideg"),
            ("2020-12-24", "Hellegowend (nomëtteg)"),
            ("2020-12-31", "Silvester"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Neijoerschdag"),
            ("2022-04-15", "Karfreideg"),
            ("2022-04-18", "Ouschterméindeg"),
            ("2022-05-01", "Dag vun der Aarbecht"),
            ("2022-05-09", "Europadag"),
            ("2022-05-26", "Christi Himmelfaart"),
            ("2022-06-06", "Péngschtméindeg"),
            ("2022-06-23", "Nationalfeierdag"),
            ("2022-08-15", "Léiffrawëschdag"),
            ("2022-11-01", "Allerhellgen"),
            ("2022-12-24", "Hellegowend (nomëtteg)"),
            ("2022-12-25", "Chrëschtdag"),
            ("2022-12-26", "Stiefesdag"),
            ("2022-12-31", "Silvester"),
        )

    def test_l10n_de(self):
        self.assertLocalizedHolidays(
            "de",
            ("2022-01-01", "Neujahr"),
            ("2022-04-15", "Karfreitag"),
            ("2022-04-18", "Ostermontag"),
            ("2022-05-01", "Tag der Arbeit"),
            ("2022-05-09", "Europatag"),
            ("2022-05-26", "Christi Himmelfahrt"),
            ("2022-06-06", "Pfingstmontag"),
            ("2022-06-23", "Nationalfeiertag"),
            ("2022-08-15", "Mariä Himmelfahrt"),
            ("2022-11-01", "Allerheiligen"),
            ("2022-12-24", "Heiligabend (Nachmittag)"),
            ("2022-12-25", "Weihnachten"),
            ("2022-12-26", "Zweiter Weihnachtsfeiertag"),
            ("2022-12-31", "Silvester"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-09", "Europe Day"),
            ("2022-05-26", "Ascension Day"),
            ("2022-06-06", "Whit Monday"),
            ("2022-06-23", "National Day"),
            ("2022-08-15", "Assumption Day"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-12-24", "Christmas Eve (afternoon)"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Saint Stephen's Day"),
            ("2022-12-31", "New Year's Eve"),
        )

    def test_l10n_fr(self):
        self.assertLocalizedHolidays(
            "fr",
            ("2022-01-01", "Jour de l'An"),
            ("2022-04-15", "Vendredi Saint"),
            ("2022-04-18", "Lundi de Pâques"),
            ("2022-05-01", "Fête du Travail"),
            ("2022-05-09", "Jour de l'Europe"),
            ("2022-05-26", "Ascension"),
            ("2022-06-06", "Lundi de Pentecôte"),
            ("2022-06-23", "Fête nationale"),
            ("2022-08-15", "Assomption"),
            ("2022-11-01", "Toussaint"),
            ("2022-12-24", "Veille de Noël (après-midi)"),
            ("2022-12-25", "Noël"),
            ("2022-12-26", "Saint Etienne"),
            ("2022-12-31", "Saint Sylvestre"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-05-01", "День праці"),
            ("2022-05-09", "День Європи"),
            ("2022-05-26", "Вознесіння Господнє"),
            ("2022-06-06", "День Святого Духа"),
            ("2022-06-23", "Національне свято"),
            ("2022-08-15", "Внебовзяття Пресвятої Діви Марії"),
            ("2022-11-01", "День усіх святих"),
            ("2022-12-24", "Святий вечір (друга половина дня)"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "День Святого Стефана"),
            ("2022-12-31", "Переддень Нового року"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.countries.france import France
from holidays.mixins.child_entity import ChildEntity


class FrenchSouthernTerritories(ChildEntity, France):
    """French Southern Territories holidays.

    Alias of a French subdivision that is also officially assigned
    its own country code in ISO 3166-1.

    !!! note "Note"
        Since most islands doesn't have a permanent population,
        the holidays are just the public ones from France.

    References:
        * <https://web.archive.org/web/20250427081803/https://www.iso.org/obp/ui/>
        * <https://en.wikipedia.org/wiki/French_Southern_and_Antarctic_Lands>
        * <https://en.wikipedia.org/wiki/Public_holidays_in_France>
    """

    country = "TF"
    parent_entity = France
    # This overseas territory was separated in 1955.
    start_year = 1956


class HolidaysTF(FrenchSouthernTerritories):
    pass


class TF(FrenchSouthernTerritories):
    pass


class ATF(FrenchSouthernTerritories):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.calendars.gregorian import DEC
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.holiday_base import HolidayBase


class EuropeanCentralBank(HolidayBase, ChristianHolidays, InternationalHolidays):
    """European Central Bank holidays.

    References:
        * <https://en.wikipedia.org/wiki/TARGET2>
        * <https://web.archive.org/web/20240615220009/https://www.ecb.europa.eu/press/pr/date/1998/html/pr980903.en.html>
        * <https://web.archive.org/web/20250403115031/https://www.ecb.europa.eu/ecb/contacts/working-hours/html/index.en.html>
        * <https://web.archive.org/web/20241109145056/https://www.ecb.europa.eu/press/pr/date/1999/html/pr990715_1.en.html>
        * <https://web.archive.org/web/20241202213944/https://www.ecb.europa.eu/press/pr/date/2000/html/pr001214_4.en.html>
        * <https://web.archive.org/web/20251216040542/https://www.ecb.europa.eu/press/pr/date/2000/html/pr000525_2.en.html>
    """

    market = "XECB"
    start_year = 1999

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, EuropeanCentralBankStaticHolidays)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        self._add_new_years_day("New Year's Day")
        self._add_christmas_day("Christmas Day")

        if self._year >= 2000:
            self._add_good_friday("Good Friday")
            self._add_easter_monday("Easter Monday")
            self._add_labor_day("Labour Day")
            self._add_christmas_day_two("Christmas Holiday")


class XECB(EuropeanCentralBank):
    pass


class ECB(EuropeanCentralBank):
    pass


class TAR(EuropeanCentralBank):
    pass


class EuropeanCentralBankStaticHolidays:
    special_public_holidays = {
        1999: (DEC, 31, "Additional closing day"),
        2001: (DEC, 31, "Additional closing day"),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import unittest
from datetime import date

from holidays.calendars.burmese import _BurmeseLunisolar


class TestBurmeseCalendar(unittest.TestCase):
    def setUp(self):
        super().setUp()
        self.calendar = _BurmeseLunisolar()

    def test_year_bounds(self):
        self.assertEqual(self.calendar.thingyan_dates(1938), (None, None))
        self.assertEqual(self.calendar.thingyan_dates(2101), (None, None))

    def test_jdn_to_gregorian(self):
        for jdn, ymd in (
            (2451544, (1999, 12, 31)),
            (2451545, (2000, 1, 1)),
            (2451604, (2000, 2, 29)),
            (2451605, (2000, 3, 1)),
            (2460677, (2025, 1, 1)),
        ):
            self.assertEqual(self.calendar.jdn_to_gregorian(jdn), date(*ymd))

    def test_new_year_dates(self):
        for year, akya_day, atat_day in (
            (1939, 14, 16),
            (1940, 13, 15),
            (1946, 13, 16),
            (1979, 14, 16),
            (1980, 13, 15),
            (1981, 13, 16),
            (2034, 14, 16),
            (2035, 14, 17),
            (2036, 14, 16),
            (2050, 14, 16),
            (2051, 15, 17),
            (2052, 14, 16),
        ):
            self.assertEqual(
                self.calendar.thingyan_dates(year),
                (date(year, 4, akya_day), date(year, 4, atat_day)),
            )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.calendars.gregorian import SAT, SUN
from holidays.constants import HALF_DAY, PUBLIC
from holidays.countries.hongkong import HongKong


class HongKongStockExchange(HongKong):
    """Hong Kong Stock Exchange (HKEX) holidays.

    References:
        * <https://web.archive.org/web/20260219133739/https://www.hkex.com.hk/Services/Trading-hours-and-Severe-Weather-Arrangements/Trading-Hours/Securities-Market?sc_lang=en>

    Historical data:
        * [2014](https://web.archive.org/web/20260329213808/https://www.hkex.com.hk/-/media/hkex-market/services/circulars-and-notices/participant-and-members-circulars/sehk/2013/ct01013e)
        * [2026](https://web.archive.org/web/20251219153754/https://www.hkex.com.hk/-/media/HKEX-Market/Services/Circulars-and-Notices/Participant-and-Members-Circulars/SEHK/2025/ce_SEHK_CT_075_2025.pdf)
    """

    country = None  # type: ignore[assignment]
    market = "XHKG"
    parent_entity = HongKong
    start_year = 2014
    supported_categories = (HALF_DAY, PUBLIC)
    weekend = {SAT, SUN}

    def _add_holiday(self, name, *args):
        if self._is_weekend(*args):
            return None

        return super()._add_holiday(name, *args)

    def _populate_public_holidays(self):
        super()._populate_optional_holidays()

    def _populate_half_day_holidays(self):
        # %s (Half-Day Trading Day).
        half_day_label = self.tr("%s（半日交易日）")

        # Chinese New Year's Eve.
        self._add_chinese_new_years_eve(half_day_label % self.tr("農曆年初一的前一日"))

        # Christmas Eve.
        self._add_christmas_eve(half_day_label % self.tr("平安夜"))

        # New Year's Eve.
        self._add_new_years_eve(half_day_label % self.tr("新年前夕"))


class XHKG(HongKongStockExchange):
    pass


class HKEX(HongKongStockExchange):
    pass


class SEHK(HongKongStockExchange):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

# mypy: disable-error-code=attr-defined

from unittest import TestCase

from holidays.calendars.gregorian import FEB, JUN, JUL, AUG, DEC
from holidays.countries.solomon_islands import SolomonIslands
from tests.common import CommonCountryTests


class TestSolomonIslands(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(SolomonIslands)

    def test_special_holidays(self):
        name = "Public Holiday"
        self.assertHolidayName(
            name,
            "2022-09-12",
            "2024-04-17",
        )

        dts = "2020-11-18"
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {"CH", "GU"}:
                self.assertHolidayName(name, holidays, dts)
            else:
                self.assertNoHoliday(holidays, dts)

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2010-12-31",
            "2012-01-02",
            "2017-01-02",
            "2021-12-31",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_holy_saturday(self):
        name = "Holy Saturday"
        self.assertHolidayName(
            name,
            "2019-04-20",
            "2020-04-11",
            "2021-04-03",
            "2022-04-16",
            "2023-04-08",
            "2024-03-30",
            "2025-04-19",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_monday(self):
        name = "Whit Monday"
        self.assertHolidayName(
            name,
            "2019-06-10",
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_queen_king_birthday(self):
        name_queen = "Queen's Birthday"
        name_king = "King's Birthday"

        self.assertHolidayName(
            name_queen,
            "2019-06-08",
            "2020-06-13",
            "2021-06-12",
            "2022-06-03",
        )
        self.assertHolidayName(name_queen, range(self.start_year, 2023))
        self.assertNoHolidayName(name_queen, range(2023, self.end_year))

        self.assertHolidayName(
            name_king,
            "2023-06-16",
            "2024-06-15",
            "2025-06-13",
        )
        self.assertHolidayName(name_king, range(2023, self.end_year))
        self.assertNoHolidayName(name_king, range(self.start_year, 2023))

        obs_dts_queen = (
            "2019-06-07",
            "2020-06-12",
            "2021-06-11",
        )
        self.assertHolidayName(f"{name_queen} (observed)", obs_dts_queen)
        obs_dts_king = ("2024-06-14",)
        self.assertHolidayName(f"{name_king} (observed)", obs_dts_king)
        self.assertNoNonObservedHoliday(obs_dts_queen, obs_dts_king)

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-07-07" for year in self.full_range))
        obs_dts = (
            "2013-07-08",
            "2019-07-08",
            "2024-07-08",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2010-12-27",
            "2011-12-27",
            "2016-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_day_of_thanksgiving(self):
        name = "National Day of Thanksgiving"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2009-12-28",
            "2010-12-28",
            "2015-12-28",
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def _test_province_day_helper(
        self,
        province_code: str,
        holiday_name: str,
        month: int,
        day: int,
        obs_dts: tuple[str, ...],
    ):
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == province_code:
                self.assertHolidayName(
                    holiday_name,
                    holidays,
                    (f"{year}-{month}-{day}" for year in self.full_range),
                )
                observed_name = f"{holiday_name} (observed)"
                self.assertHolidayName(observed_name, holidays, obs_dts)
                self.assertNoNonObservedHolidayName(
                    observed_name,
                    self.subdiv_holidays_non_observed[subdiv],
                    obs_dts,
                )
            else:
                self.assertNoHolidayName(holiday_name, holidays, self.full_range)

    def test_central_province_day(self):
        obs_dts = (
            "2013-06-28",
            "2014-06-30",
            "2019-06-28",
            "2024-06-28",
            "2025-06-30",
        )
        self._test_province_day_helper("CE", "Central Province Day", JUN, 29, obs_dts)

    def test_choiseul_province_day(self):
        obs_dts = (
            "2012-02-24",
            "2017-02-24",
            "2018-02-26",
            "2023-02-24",
            "2024-02-26",
        )
        self._test_province_day_helper("CH", "Choiseul Province Day", FEB, 25, obs_dts)

    def test_guadalcanal_province_day(self):
        obs_dts = (
            "2010-08-02",
            "2015-07-31",
            "2020-07-31",
            "2021-08-02",
            "2026-07-31",
        )
        self._test_province_day_helper("GU", "Guadalcanal Province Day", AUG, 1, obs_dts)

    def test_isabel_province_day(self):
        obs_dts = (
            "2012-06-01",
            "2013-06-03",
            "2018-06-01",
            "2019-06-03",
            "2024-06-03",
        )
        self._test_province_day_helper("IS", "Isabel Province Day", JUN, 2, obs_dts)

    def test_makira_ulawa_province_day(self):
        obs_dts = (
            "2013-08-02",
            "2014-08-04",
            "2019-08-02",
            "2024-08-02",
            "2025-08-04",
        )
        self._test_province_day_helper("MK", "Makira-Ulawa Province Day", AUG, 3, obs_dts)

    def test_malaita_province_day(self):
        obs_dts = (
            "2010-08-16",
            "2015-08-14",
            "2020-08-14",
            "2021-08-16",
            "2026-08-14",
        )
        self._test_province_day_helper("ML", "Malaita Province Day", AUG, 15, obs_dts)

    def test_rennell_and_bellona_province_day(self):
        obs_dts = (
            "2013-07-19",
            "2014-07-21",
            "2019-07-19",
            "2024-07-19",
            "2025-07-21",
        )
        self._test_province_day_helper("RB", "Rennell and Bellona Province Day", JUL, 20, obs_dts)

    def test_temotu_province_day(self):
        obs_dts = (
            "2013-06-07",
            "2014-06-09",
            "2019-06-07",
            "2024-06-07",
            "2025-06-09",
        )
        self._test_province_day_helper("TE", "Temotu Province Day", JUN, 8, obs_dts)

    def test_western_province_day(self):
        obs_dts = (
            "2013-12-06",
            "2014-12-08",
            "2019-12-06",
            "2024-12-06",
            "2025-12-08",
        )
        self._test_province_day_helper("WE", "Western Province Day", DEC, 7, obs_dts)

    def test_2025(self):
        self.assertHolidaysInYear(
            2025,
            ("2025-01-01", "New Year's Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-19", "Holy Saturday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-06-09", "Whit Monday"),
            ("2025-06-13", "King's Birthday"),
            ("2025-07-07", "Independence Day"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "National Day of Thanksgiving"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.kuwait import Kuwait
from tests.common import CommonCountryTests


class TestKuwait(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Kuwait)

    def test_new_years_day(self):
        self.assertHolidayName(
            "رأس السنة الميلادية", (f"{year}-01-01" for year in self.full_range)
        )

    def test_national_day(self):
        self.assertHolidayName("اليوم الوطني", (f"{year}-02-25" for year in self.full_range))

    def test_liberation_day(self):
        self.assertHolidayName("يوم التحرير", (f"{year}-02-26" for year in self.full_range))

    def test_islamic_new_year(self):
        name = "رأس السنة الهجرية"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-08-20",
            "2021-08-09",
            "2022-07-30",
            "2023-07-19",
            "2024-07-07",
            "2025-06-26",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_prophets_birthday(self):
        name = "عيد المولد النبوي"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
            "2023-09-27",
            "2024-09-15",
            "2025-09-04",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_isra_and_miraj(self):
        name = "ليلة المعراج"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-03-22",
            "2021-03-11",
            "2022-02-28",
            "2023-02-18",
            "2024-02-08",
            "2025-01-27",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_fitr(self):
        name = "عيد الفطر"
        name_holiday = f"عطلة {name}"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

        self.assertIslamicNoEstimatedHolidayName(
            name_holiday,
            "2020-05-25",
            "2020-05-26",
            "2021-05-14",
            "2021-05-15",
            "2022-05-03",
            "2022-05-04",
            "2023-04-22",
            "2023-04-23",
            "2024-04-11",
            "2024-04-12",
            "2025-03-31",
            "2025-04-01",
        )
        self.assertIslamicNoEstimatedHolidayName(name_holiday, self.full_range)

    def test_arafat_day(self):
        name = "يوم عرفة"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-07-30",
            "2021-07-19",
            "2022-07-08",
            "2023-06-27",
            "2024-06-15",
            "2025-06-05",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "عيد الأضحى"
        name_holiday = f"عطلة {name}"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

        self.assertIslamicNoEstimatedHolidayName(
            name_holiday,
            "2020-08-01",
            "2020-08-02",
            "2021-07-21",
            "2021-07-22",
            "2022-07-10",
            "2022-07-11",
            "2023-06-29",
            "2023-06-30",
            "2024-06-17",
            "2024-06-18",
            "2025-06-07",
            "2025-06-08",
        )
        self.assertIslamicNoEstimatedHolidayName(name_holiday, self.full_range)

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "رأس السنة الميلادية"),
            ("2022-02-25", "اليوم الوطني"),
            ("2022-02-26", "يوم التحرير"),
            ("2022-02-28", "ليلة المعراج (تقديري)"),
            ("2022-05-02", "عيد الفطر (تقديري)"),
            ("2022-05-03", "عطلة عيد الفطر (تقديري)"),
            ("2022-05-04", "عطلة عيد الفطر (تقديري)"),
            ("2022-07-08", "يوم عرفة (تقديري)"),
            ("2022-07-09", "عيد الأضحى (تقديري)"),
            ("2022-07-10", "عطلة عيد الأضحى (تقديري)"),
            ("2022-07-11", "عطلة عيد الأضحى (تقديري)"),
            ("2022-07-30", "رأس السنة الهجرية (تقديري)"),
            ("2022-10-08", "عيد المولد النبوي (تقديري)"),
        )

    def test_weekend(self):
        for dt in (
            "2007-08-30",  # THU.
            "2007-08-31",  # FRI.
            "2007-09-01",  # SAT.
            "2007-09-07",  # FRI.
            "2007-09-08",  # SAT.
        ):
            self.assertTrue(self.holidays.is_weekend(dt))

        for dt in (
            "2007-08-25",  # SAT.
            "2007-08-26",  # SUN.
            "2007-09-02",  # SUN.
            "2007-09-06",  # THU.
        ):
            self.assertFalse(self.holidays.is_weekend(dt))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "رأس السنة الميلادية"),
            ("2023-02-18", "ليلة المعراج (تقديري)"),
            ("2023-02-25", "اليوم الوطني"),
            ("2023-02-26", "يوم التحرير"),
            ("2023-04-21", "عيد الفطر (تقديري)"),
            ("2023-04-22", "عطلة عيد الفطر (تقديري)"),
            ("2023-04-23", "عطلة عيد الفطر (تقديري)"),
            ("2023-06-27", "يوم عرفة (تقديري)"),
            ("2023-06-28", "عيد الأضحى (تقديري)"),
            ("2023-06-29", "عطلة عيد الأضحى (تقديري)"),
            ("2023-06-30", "عطلة عيد الأضحى (تقديري)"),
            ("2023-07-19", "رأس السنة الهجرية (تقديري)"),
            ("2023-09-27", "عيد المولد النبوي (تقديري)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-02-18", "Isra' and Mi'raj (estimated)"),
            ("2023-02-25", "National Day"),
            ("2023-02-26", "Liberation Day"),
            ("2023-04-21", "Eid al-Fitr (estimated)"),
            ("2023-04-22", "Eid al-Fitr Holiday (estimated)"),
            ("2023-04-23", "Eid al-Fitr Holiday (estimated)"),
            ("2023-06-27", "Arafat Day (estimated)"),
            ("2023-06-28", "Eid al-Adha (estimated)"),
            ("2023-06-29", "Eid al-Adha Holiday (estimated)"),
            ("2023-06-30", "Eid al-Adha Holiday (estimated)"),
            ("2023-07-19", "Islamic New Year (estimated)"),
            ("2023-09-27", "Prophet's Birthday (estimated)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SAT_TO_NONE, SUN_TO_NEXT_MON


class IceFuturesEurope(ObservedHolidayBase, ChristianHolidays, InternationalHolidays):
    """ICE Futures Europe holidays.

    References:
        * <https://web.archive.org/web/20241203121313/https://www.ice.com/publicdocs/futures/Trading_Schedule_Migrated_Liffe_Contracts.pdf>
        * <https://web.archive.org/web/20241118133442/https://www.ice.com/publicdocs/Trading_Schedule.pdf>
        * <https://web.archive.org/web/20230927015846/https://www.ice.com/publicdocs/Trading_Schedule.pdf>
        * <https://web.archive.org/web/20211022183728/https://www.ice.com/publicdocs/Trading_Schedule.pdf>
    """

    market = "IFEU"
    start_year = 2014

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        kwargs.setdefault("observed_rule", SAT_TO_NONE + SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        self._move_holiday(self._add_new_years_day("New Year's Day"))

        self._add_good_friday("Good Friday")

        self._move_holiday(self._add_christmas_day("Christmas Day"))


class ICEFuturesEurope(IceFuturesEurope):
    pass


class IFEU(IceFuturesEurope):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.palestine import Palestine
from tests.common import CommonCountryTests


class TestPalestine(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Palestine)

    def test_new_years_day(self):
        name = "رأس السنة الميلادي"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        self.assertCatholicHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        self.assertNoOrthodoxHolidayName(name)

    def test_womens_day(self):
        self.assertHolidayName("يوم المراة العالمي", (f"{year}-03-08" for year in self.full_range))

    def test_labor_day(self):
        self.assertHolidayName("عيد العمال", (f"{year}-05-01" for year in self.full_range))

    def test_independence_day(self):
        self.assertHolidayName("عيد الإستقلال", (f"{year}-11-15" for year in self.full_range))

    def test_christmas_orthodox(self):
        name = "عيد الميلاد المجيد الشرقي"
        self.assertHolidayName(name, (f"{year}-01-07" for year in self.full_range))
        self.assertOrthodoxHolidayName(name, (f"{year}-01-08" for year in self.full_range))

    def test_new_years_day_orthodox(self):
        name = "عيد رأس السنة الشرقي"
        self.assertNoHolidayName(name)
        self.assertNoCatholicHolidayName(name)
        self.assertOrthodoxHolidayName(name, (f"{year}-01-14" for year in self.full_range))

    def test_epiphany_orthodox(self):
        name = "عيد الغطاس"
        self.assertNoHolidayName(name)
        self.assertOrthodoxHolidayName(name, (f"{year}-01-19" for year in self.full_range))

    def test_palm_sunday_orthodox(self):
        name = "أحد الشعانين"
        self.assertNoHolidayName(name)
        self.assertOrthodoxHolidayName(
            name,
            "2020-04-12",
            "2021-04-25",
            "2022-04-17",
            "2023-04-09",
            "2024-04-28",
            "2025-04-13",
        )
        self.assertOrthodoxHolidayName(name, self.full_range)

    def test_holy_thursday_orthodox(self):
        name = "خميس الغسل"
        self.assertNoHolidayName(name)
        self.assertOrthodoxHolidayName(
            name,
            "2020-04-16",
            "2021-04-29",
            "2022-04-21",
            "2023-04-13",
            "2024-05-02",
            "2025-04-17",
        )
        self.assertOrthodoxHolidayName(name, self.full_range)

    def test_good_friday_orthodox(self):
        name = "الجمعة العظيمة"
        self.assertNoHolidayName(name)
        self.assertOrthodoxHolidayName(
            name,
            "2020-04-17",
            "2021-04-30",
            "2022-04-22",
            "2023-04-14",
            "2024-05-03",
            "2025-04-18",
        )
        self.assertOrthodoxHolidayName(name, self.full_range)

    def test_holy_saturday_orthodox(self):
        name = "سبت النور"
        self.assertNoHolidayName(name)
        self.assertOrthodoxHolidayName(
            name,
            "2020-04-18",
            "2021-05-01",
            "2022-04-23",
            "2023-04-15",
            "2024-05-04",
            "2025-04-19",
        )
        self.assertOrthodoxHolidayName(name, self.full_range)

    def test_easter_sunday_orthodox(self):
        name = "عيد الفصح المجيد"
        self.assertHolidayName(
            name,
            "2020-04-19",
            "2021-05-02",
            "2022-04-24",
            "2023-04-16",
            "2024-05-05",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday_orthodox(self):
        name = "عيد الفصح المجيد"
        self.assertOrthodoxHolidayName(
            name,
            "2020-04-20",
            "2021-05-03",
            "2022-04-25",
            "2023-04-17",
            "2024-05-06",
            "2025-04-21",
        )
        self.assertOrthodoxHolidayName(name, self.full_range)

    def test_ascension_thursday_orthodox(self):
        name = "خميس الصعود"
        self.assertNoHolidayName(name)
        self.assertOrthodoxHolidayName(
            name,
            "2020-05-28",
            "2021-06-10",
            "2022-06-02",
            "2023-05-25",
            "2024-06-13",
            "2025-05-29",
        )
        self.assertOrthodoxHolidayName(name, self.full_range)

    def test_pentecost_orthodox(self):
        name = "أحد العنصرة"
        self.assertNoHolidayName(name)
        self.assertOrthodoxHolidayName(
            name,
            "2020-06-07",
            "2021-06-20",
            "2022-06-12",
            "2023-06-04",
            "2024-06-23",
            "2025-06-08",
        )
        self.assertOrthodoxHolidayName(name, self.full_range)

    def test_epiphany_catholic(self):
        name = "عيد الغطاس"
        self.assertNoHolidayName(name)
        self.assertCatholicHolidayName(name, (f"{year}-01-06" for year in self.full_range))

    def test_palm_sunday_catholic(self):
        name = "أحد الشعانين"
        self.assertNoHolidayName(name)
        self.assertCatholicHolidayName(
            name,
            "2020-04-05",
            "2021-03-28",
            "2022-04-10",
            "2023-04-02",
            "2024-03-24",
            "2025-04-13",
        )
        self.assertCatholicHolidayName(name, self.full_range)

    def test_holy_thursday_catholic(self):
        name = "خميس الغسل"
        self.assertNoHolidayName(name)
        self.assertCatholicHolidayName(
            name,
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
            "2024-03-28",
            "2025-04-17",
        )
        self.assertCatholicHolidayName(name, self.full_range)

    def test_good_friday_catholic(self):
        name = "الجمعة العظيمة"
        self.assertNoHolidayName(name)
        self.assertCatholicHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertCatholicHolidayName(name, self.full_range)

    def test_holy_saturday_catholic(self):
        name = "سبت النور"
        self.assertNoHolidayName(name)
        self.assertCatholicHolidayName(
            name,
            "2020-04-11",
            "2021-04-03",
            "2022-04-16",
            "2023-04-08",
            "2024-03-30",
            "2025-04-19",
        )
        self.assertCatholicHolidayName(name, self.full_range)

    def test_easter_sunday_catholic(self):
        name = "عيد الفصح المجيد"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday_catholic(self):
        name = "عيد الفصح المجيد"
        self.assertCatholicHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertCatholicHolidayName(name, self.full_range)

    def test_ascension_thursday_catholic(self):
        name = "خميس الصعود"
        self.assertNoHolidayName(name)
        self.assertCatholicHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertCatholicHolidayName(name, self.full_range)

    def test_pentecost_catholic(self):
        name = "أحد العنصرة"
        self.assertNoHolidayName(name)
        self.assertCatholicHolidayName(
            name,
            "2020-05-31",
            "2021-05-23",
            "2022-06-05",
            "2023-05-28",
            "2024-05-19",
            "2025-06-08",
        )
        self.assertCatholicHolidayName(name, self.full_range)

    def test_christmas_catholic(self):
        name = "عيد الميلاد المجيد الغربي"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        self.assertCatholicHolidayName(name, (f"{year}-12-26" for year in self.full_range))

    def test_hijri_new_year(self):
        name = "رأس السنة الهجرية"
        self.assertHolidayName(
            name,
            "2020-08-20",
            "2021-08-09",
            "2022-07-30",
            "2023-07-19",
            "2024-07-07",
            "2025-06-26",
        )
        self.assertHolidayName(name, self.full_range)

    def test_mawlid(self):
        name = "ذكرى المولد النبوي الشريف"
        self.assertHolidayName(
            name,
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
            "2023-09-27",
            "2024-09-15",
        )
        self.assertHolidayName(name, self.full_range)

    def test_isra_and_miraj(self):
        name = "ذكرى الإسراء والمعراج"
        self.assertHolidayName(
            name,
            "2020-03-22",
            "2021-03-11",
            "2022-02-28",
            "2023-02-18",
            "2024-02-08",
            "2025-01-27",
        )
        self.assertHolidayName(name, self.full_range)

    def test_eid_al_fitr(self):
        name = "عيد الفطر السعيد"
        self.assertHolidayName(
            name,
            "2020-05-24",
            "2020-05-25",
            "2020-05-26",
            "2021-05-13",
            "2021-05-14",
            "2021-05-15",
            "2022-05-02",
            "2022-05-03",
            "2022-05-04",
            "2023-04-21",
            "2023-04-22",
            "2023-04-23",
            "2024-04-10",
            "2024-04-11",
            "2024-04-12",
            "2025-03-30",
            "2025-03-31",
            "2025-04-01",
        )
        self.assertHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "عيد الأضحى المبارك"
        self.assertHolidayName(
            name,
            "2020-07-31",
            "2020-08-01",
            "2020-08-02",
            "2020-08-03",
            "2021-07-20",
            "2021-07-21",
            "2021-07-22",
            "2021-07-23",
            "2022-07-09",
            "2022-07-10",
            "2022-07-11",
            "2022-07-12",
            "2023-06-28",
            "2023-06-29",
            "2023-06-30",
            "2023-07-01",
            "2024-06-16",
            "2024-06-17",
            "2024-06-18",
            "2024-06-19",
            "2025-06-06",
            "2025-06-07",
            "2025-06-08",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-01-06", "Epiphany"),
            ("2024-01-07", "Orthodox Christmas Day"),
            ("2024-01-08", "Orthodox Christmas Day"),
            ("2024-01-14", "Orthodox New Year's Day"),
            ("2024-01-19", "Epiphany"),
            ("2024-02-08", "Isra' and Mi'raj"),
            ("2024-03-08", "International Women's Day"),
            ("2024-03-24", "Palm Sunday"),
            ("2024-03-28", "Holy Thursday"),
            ("2024-03-29", "Good Friday"),
            ("2024-03-30", "Holy Saturday"),
            ("2024-03-31", "Easter"),
            ("2024-04-01", "Easter"),
            ("2024-04-10", "Eid al-Fitr"),
            ("2024-04-11", "Eid al-Fitr"),
            ("2024-04-12", "Eid al-Fitr"),
            ("2024-04-28", "Palm Sunday"),
            ("2024-05-01", "Labor Day"),
            ("2024-05-02", "Holy Thursday"),
            ("2024-05-03", "Good Friday"),
            ("2024-05-04", "Holy Saturday"),
            ("2024-05-05", "Easter"),
            ("2024-05-06", "Easter"),
            ("2024-05-09", "Ascension Day"),
            ("2024-05-19", "Pentecost"),
            ("2024-06-13", "Ascension Day"),
            ("2024-06-16", "Eid al-Adha"),
            ("2024-06-17", "Eid al-Adha"),
            ("2024-06-18", "Eid al-Adha"),
            ("2024-06-19", "Eid al-Adha"),
            ("2024-06-23", "Pentecost"),
            ("2024-07-07", "Hijri New Year"),
            ("2024-09-15", "Prophet's Birthday"),
            ("2024-11-15", "Independence Day"),
            ("2024-12-25", "Catholic Christmas Day"),
            ("2024-12-26", "Catholic Christmas Day"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "رأس السنة الميلادي"),
            ("2024-01-06", "عيد الغطاس"),
            ("2024-01-07", "عيد الميلاد المجيد الشرقي"),
            ("2024-01-08", "عيد الميلاد المجيد الشرقي"),
            ("2024-01-14", "عيد رأس السنة الشرقي"),
            ("2024-01-19", "عيد الغطاس"),
            ("2024-02-08", "ذكرى الإسراء والمعراج"),
            ("2024-03-08", "يوم المراة العالمي"),
            ("2024-03-24", "أحد الشعانين"),
            ("2024-03-28", "خميس الغسل"),
            ("2024-03-29", "الجمعة العظيمة"),
            ("2024-03-30", "سبت النور"),
            ("2024-03-31", "عيد الفصح المجيد"),
            ("2024-04-01", "عيد الفصح المجيد"),
            ("2024-04-10", "عيد الفطر السعيد"),
            ("2024-04-11", "عيد الفطر السعيد"),
            ("2024-04-12", "عيد الفطر السعيد"),
            ("2024-04-28", "أحد الشعانين"),
            ("2024-05-01", "عيد العمال"),
            ("2024-05-02", "خميس الغسل"),
            ("2024-05-03", "الجمعة العظيمة"),
            ("2024-05-04", "سبت النور"),
            ("2024-05-05", "عيد الفصح المجيد"),
            ("2024-05-06", "عيد الفصح المجيد"),
            ("2024-05-09", "خميس الصعود"),
            ("2024-05-19", "أحد العنصرة"),
            ("2024-06-13", "خميس الصعود"),
            ("2024-06-16", "عيد الأضحى المبارك"),
            ("2024-06-17", "عيد الأضحى المبارك"),
            ("2024-06-18", "عيد الأضحى المبارك"),
            ("2024-06-19", "عيد الأضحى المبارك"),
            ("2024-06-23", "أحد العنصرة"),
            ("2024-07-07", "رأس السنة الهجرية"),
            ("2024-09-15", "ذكرى المولد النبوي الشريف"),
            ("2024-11-15", "عيد الإستقلال"),
            ("2024-12-25", "عيد الميلاد المجيد الغربي"),
            ("2024-12-26", "عيد الميلاد المجيد الغربي"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.western_sahara import WesternSahara
from tests.common import CommonCountryTests


class TestWesternSahara(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        cls.full_range = range(2000, 2050)
        super().setUpClass(WesternSahara, years=cls.full_range)
        cls.no_estimated_holidays = WesternSahara(
            years=cls.full_range, islamic_show_estimated=False
        )

    def test_proclamation_of_the_sadr(self):
        self.assertHolidayName(
            "إعلان الجمهورية العربية الصحراوية الديمقراطية",
            (f"{year}-02-27" for year in self.full_range),
        )

    def test_first_martyr(self):
        self.assertHolidayName("يوم الشهيد الأول", (f"{year}-03-08" for year in self.full_range))

    def test_creation_of_the_polisario_front(self):
        self.assertHolidayName(
            "تأسيس الجبهة الشعبية لتحرير الساقية الحمراء ووادي الذهب",
            (f"{year}-05-10" for year in self.full_range),
        )

    def test_commencement_of_the_armed_struggle(self):
        self.assertHolidayName(
            "اندلاع الكفاح المسلح", (f"{year}-05-20" for year in self.full_range)
        )

    def test_martyrs_day(self):
        self.assertHolidayName("يوم الشهداء", (f"{year}-06-09" for year in self.full_range))

    def test_uprising_day(self):
        self.assertHolidayName("يوم الانتفاضة", (f"{year}-06-17" for year in self.full_range))

    def test_national_unity_day(self):
        self.assertHolidayName("عيد الوحدة الوطنية", (f"{year}-10-12" for year in self.full_range))

    def test_islamic_new_year(self):
        name = "رأس السنة الهجرية"
        self.assertHolidayName(
            name,
            self.no_estimated_holidays,
            "2020-08-20",
            "2021-08-09",
            "2022-07-30",
            "2023-07-19",
            "2024-07-07",
            "2025-06-26",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(2004, 2050))
        self.assertNoHolidayName(name, self.no_estimated_holidays, range(2000, 2004))

    def test_prophets_birthday(self):
        name = "المولد النبوي الشريف"
        self.assertHolidayName(
            name,
            self.no_estimated_holidays,
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
            "2023-09-27",
            "2024-09-15",
            "2025-09-04",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(2004, 2050))
        self.assertNoHolidayName(name, self.no_estimated_holidays, range(2000, 2004))

    def test_eid_al_fitr(self):
        name = "عيد الفطر المبارك"
        self.assertHolidayName(
            name,
            self.no_estimated_holidays,
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(2003, 2050))
        self.assertNoHolidayName(name, self.no_estimated_holidays, range(2000, 2003))

    def test_eid_al_adha(self):
        name = "عيد الأضحى المبارك"
        self.assertHolidayName(
            name,
            self.no_estimated_holidays,
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(2004, 2050))
        self.assertNoHolidayName(name, self.no_estimated_holidays, range(2000, 2004))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-02-27", "إعلان الجمهورية العربية الصحراوية الديمقراطية"),
            ("2024-03-08", "يوم الشهيد الأول"),
            ("2024-04-10", "عيد الفطر المبارك (تقديري)"),
            ("2024-05-10", "تأسيس الجبهة الشعبية لتحرير الساقية الحمراء ووادي الذهب"),
            ("2024-05-20", "اندلاع الكفاح المسلح"),
            ("2024-06-09", "يوم الشهداء"),
            ("2024-06-16", "عيد الأضحى المبارك (تقديري)"),
            ("2024-06-17", "يوم الانتفاضة"),
            ("2024-07-07", "رأس السنة الهجرية (تقديري)"),
            ("2024-09-15", "المولد النبوي الشريف (تقديري)"),
            ("2024-10-12", "عيد الوحدة الوطنية"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-02-27", "Proclamation of the SADR"),
            ("2024-03-08", "First Martyr"),
            ("2024-04-10", "Eid al-Fitr (estimated)"),
            ("2024-05-10", "Creation of the Polisario Front"),
            ("2024-05-20", "Commencement of the Armed Struggle"),
            ("2024-06-09", "Martyrs' Day"),
            ("2024-06-16", "Eid al-Adha (estimated)"),
            ("2024-06-17", "Uprising Day"),
            ("2024-07-07", "Islamic New Year (estimated)"),
            ("2024-09-15", "Prophet's Birthday (estimated)"),
            ("2024-10-12", "National Unity Day"),
        )

    def test_l10n_es(self):
        self.assertLocalizedHolidays(
            "es",
            ("2024-02-27", "Proclamación de la República Árabe Saharaui Democrática"),
            ("2024-03-08", "Primer mártir"),
            ("2024-04-10", "Eid al-Fitr (estimado)"),
            ("2024-05-10", "Creación del Frente POLISARIO"),
            ("2024-05-20", "Inicio de la lucha armada"),
            ("2024-06-09", "Día de los mártires"),
            ("2024-06-16", "Eid al-Adha (estimado)"),
            ("2024-06-17", "Día de la insurrección"),
            ("2024-07-07", "Primer día del año de la Hégira (estimado)"),
            ("2024-09-15", "Santo Nacimiento del Profeta (estimado)"),
            ("2024-10-12", "Fiesta de la unidad nacional"),
        )

    def test_l10n_fr(self):
        self.assertLocalizedHolidays(
            "fr",
            ("2024-02-27", "Proclamation de la République arabe sahraouie démocratique"),
            ("2024-03-08", "Premier martyr"),
            ("2024-04-10", "Aïd el-Fitr (estimé)"),
            ("2024-05-10", "Création du Front POLISARIO"),
            ("2024-05-20", "Déclenchement de la lutte armée"),
            ("2024-06-09", "Journée des martyrs"),
            ("2024-06-16", "Aïd al-Adha (estimé)"),
            ("2024-06-17", "Jour de l'insurrection"),
            ("2024-07-07", "Premier jour de l'an de l'Hégire (estimé)"),
            ("2024-09-15", "Naissance du Prophète (estimé)"),
            ("2024-10-12", "Fête de l'unité nationale"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.malta import Malta
from tests.common import CommonCountryTests


class TestMalta(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Malta)

    def test_1980(self):
        self.assertHolidaysInYear(
            1980,
            ("1980-01-01", "L-Ewwel tas-Sena"),
            ("1980-03-31", "Jum il-Ħelsien"),
            ("1980-04-04", "Il-Ġimgħa l-Kbira"),
            ("1980-05-01", "Jum il-Ħaddiem"),
            ("1980-08-15", "Il-Festa ta' Santa Marija"),
            ("1980-12-13", "Jum ir-Repubblika"),
            ("1980-12-25", "Il-Milied"),
        )

    def test_1987(self):
        self.assertHolidaysInYear(
            1987,
            ("1987-01-01", "L-Ewwel tas-Sena"),
            ("1987-02-10", "Il-Festa tan-Nawfraġju ta' San Pawl"),
            ("1987-03-19", "Il-Festa ta' San Ġużepp"),
            ("1987-04-17", "Il-Ġimgħa l-Kbira"),
            ("1987-05-01", "Jum il-Ħaddiem"),
            ("1987-06-29", "Il-Festa ta' San Pietru u San Pawl"),
            ("1987-08-15", "Il-Festa ta' Santa Marija"),
            ("1987-09-08", "Jum il-Vitorja"),
            ("1987-09-21", "Jum l-Indipendenza"),
            ("1987-12-08", "Il-Festa tal-Immakulata Kunċizzjoni"),
            ("1987-12-13", "Jum ir-Repubblika"),
            ("1987-12-25", "Il-Milied"),
        )

    def test_1989(self):
        self.assertHolidaysInYear(
            1989,
            ("1989-01-01", "L-Ewwel tas-Sena"),
            ("1989-02-10", "Il-Festa tan-Nawfraġju ta' San Pawl"),
            ("1989-03-19", "Il-Festa ta' San Ġużepp"),
            ("1989-03-24", "Il-Ġimgħa l-Kbira"),
            ("1989-03-31", "Jum il-Ħelsien"),
            ("1989-05-01", "Jum il-Ħaddiem"),
            ("1989-06-07", "Sette Giugno"),
            ("1989-06-29", "Il-Festa ta' San Pietru u San Pawl"),
            ("1989-08-15", "Il-Festa ta' Santa Marija"),
            ("1989-09-08", "Jum il-Vitorja"),
            ("1989-09-21", "Jum l-Indipendenza"),
            ("1989-12-08", "Il-Festa tal-Immakulata Kunċizzjoni"),
            ("1989-12-13", "Jum ir-Repubblika"),
            ("1989-12-25", "Il-Milied"),
        )

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "L-Ewwel tas-Sena"),
            ("2022-02-10", "Il-Festa tan-Nawfraġju ta' San Pawl"),
            ("2022-03-19", "Il-Festa ta' San Ġużepp"),
            ("2022-03-31", "Jum il-Ħelsien"),
            ("2022-04-15", "Il-Ġimgħa l-Kbira"),
            ("2022-05-01", "Jum il-Ħaddiem"),
            ("2022-06-07", "Sette Giugno"),
            ("2022-06-29", "Il-Festa ta' San Pietru u San Pawl"),
            ("2022-08-15", "Il-Festa ta' Santa Marija"),
            ("2022-09-08", "Jum il-Vitorja"),
            ("2022-09-21", "Jum l-Indipendenza"),
            ("2022-12-08", "Il-Festa tal-Immakulata Kunċizzjoni"),
            ("2022-12-13", "Jum ir-Repubblika"),
            ("2022-12-25", "Il-Milied"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "L-Ewwel tas-Sena"),
            ("2023-02-10", "Il-Festa tan-Nawfraġju ta' San Pawl"),
            ("2023-03-19", "Il-Festa ta' San Ġużepp"),
            ("2023-03-31", "Jum il-Ħelsien"),
            ("2023-04-07", "Il-Ġimgħa l-Kbira"),
            ("2023-05-01", "Jum il-Ħaddiem"),
            ("2023-06-07", "Sette Giugno"),
            ("2023-06-29", "Il-Festa ta' San Pietru u San Pawl"),
            ("2023-08-15", "Il-Festa ta' Santa Marija"),
            ("2023-09-08", "Jum il-Vitorja"),
            ("2023-09-21", "Jum l-Indipendenza"),
            ("2023-12-08", "Il-Festa tal-Immakulata Kunċizzjoni"),
            ("2023-12-13", "Jum ir-Repubblika"),
            ("2023-12-25", "Il-Milied"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-02-10", "Feast of Saint Paul's Shipwreck"),
            ("2023-03-19", "Feast of Saint Joseph"),
            ("2023-03-31", "Freedom Day"),
            ("2023-04-07", "Good Friday"),
            ("2023-05-01", "Worker's Day"),
            ("2023-06-07", "Sette Giugno"),
            ("2023-06-29", "Feast of Saint Peter and Saint Paul"),
            ("2023-08-15", "Feast of the Assumption"),
            ("2023-09-08", "Feast of Our Lady of Victories"),
            ("2023-09-21", "Independence Day"),
            ("2023-12-08", "Feast of the Immaculate Conception"),
            ("2023-12-13", "Republic Day"),
            ("2023-12-25", "Christmas Day"),
        )



#!/usr/bin/env python3

#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

"""Generate Germany school holidays runtime data from official KMK sources.

Workflow:
1. Run ``python scripts/calendar/germany_school_generator.py``.
2. On cold start, the script downloads official KMK school-year PDFs into a local cache
   directory outside the repository.
3. The script writes fresh data to ``holidays/calendars/germany_school_dates.py`` - a
   throwaway file that is **not committed**. It mirrors the structure of the committed
   module ``holidays/calendars/germany_school.py`` so you can diff the two directly::

       diff holidays/calendars/germany_school{_dates,}.py

4. Apply the relevant changes from ``germany_school_dates.py`` to ``germany_school.py``
   (the committed module that ships with the library).
5. Run Germany tests and docs tests, then commit the updated ``germany_school.py``.

The raw KMK PDFs and the generated ``germany_school_dates.py`` are dev-only artifacts and
are intentionally not committed. The generator keeps runtime logic in
``holidays/countries/germany.py`` small and offline.
"""

from __future__ import annotations

import argparse
import re
from collections import defaultdict
from datetime import date
from html.parser import HTMLParser
from pathlib import Path
from tempfile import gettempdir
from urllib.parse import urljoin, urlparse
from urllib.request import urlopen

ROOT_DIR = Path(__file__).resolve().parents[2]
DEFAULT_KMK_PAGE_URL = "https://www.kmk.org/service/ferienregelung.html"
DEFAULT_RAW_PDF_DIR = Path(gettempdir()) / "holidays-germany-school-holidays" / "pdfs"
OUTPUT_PATH = ROOT_DIR / "holidays" / "calendars" / "germany_school_dates.py"
SUPPORTED_START_YEAR = 1990
URL_TIMEOUT_SECONDS = 30

# KMK PDF schema adapter:
# - state labels are mapped to library subdivision codes;
# - holiday column headers are normalized to stable internal keys;
# - internal keys are mapped to canonical holiday names stored in the runtime dataset.
STATE_LABEL_TO_CODE = {
    "Baden-Württemberg": "BW",
    "Bayern": "BY",
    "Berlin": "BE",
    "Brandenburg": "BB",
    "Bremen": "HB",
    "Hamburg": "HH",
    "Hessen": "HE",
    "Mecklenburg-Vorpommern": "MV",
    "Niedersachsen": "NI",
    "Nordrhein-Westfalen": "NW",
    "Rheinland-Pfalz": "RP",
    "Saarland": "SL",
    "Sachsen": "SN",
    "Sachsen-Anhalt": "ST",
    "Schleswig-Holstein": "SH",
    "Thüringen": "TH",
}

CANONICAL_COLUMN_KEYS = {
    "Herbst": "Herbst",
    "Sommer": "Sommer",
    "Weihnachten": "Weihnachten",
    "Winter": "Winter",
    "Ostern/Frühjahr": "Ostern/Frühjahr",
    "Himmelf./Pfingsten": "Himmelfahrt/Pfingsten",
    "Himmelfahrt/Pfingsten": "Himmelfahrt/Pfingsten",
}

AUTUMN_BREAK = "AUTUMN_BREAK"
CHRISTMAS_BREAK = "CHRISTMAS_BREAK"
WINTER_BREAK = "WINTER_BREAK"
EASTER_SPRING_BREAK = "EASTER_SPRING_BREAK"
ASCENSION_WHIT_BREAK = "ASCENSION_WHIT_BREAK"
SUMMER_BREAK = "SUMMER_BREAK"

HOLIDAY_IDS = {
    "Herbst": AUTUMN_BREAK,
    "Sommer": SUMMER_BREAK,
    "Weihnachten": CHRISTMAS_BREAK,
    "Winter": WINTER_BREAK,
    "Ostern/Frühjahr": EASTER_SPRING_BREAK,
    "Himmelfahrt/Pfingsten": ASCENSION_WHIT_BREAK,
}
EXPECTED_SUBDIVISIONS = frozenset(STATE_LABEL_TO_CODE.values())

DATE_TOKEN_RE = re.compile(
    r"""
    (?P<range_full>\d{1,2}\.\d{1,2}\.?\s*-\s*\d{1,2}\.\d{1,2}\.?)
    |(?P<range_same_month>\d{1,2}\.?\s*-\s*\d{1,2}\.\d{1,2}\.?)
    |(?P<single>\d{1,2}\.\d{1,2}\.?)
    """,
    re.VERBOSE,
)
KMK_SCHOOL_YEAR_RE = re.compile(r"(?P<start>\d{4})/(?P<end>\d{2,4})")
PDF_FILENAME_YEAR_RE = re.compile(r"(?i)(?:fer)(?P<start>\d{2,4})[-_](?P<end>\d{2,4})\.pdf$")
NON_DATE_FOOTNOTE_RE = re.compile(r"(?<![\d.])\d+\)")
DATE_DAY_MONTH_FOOTNOTE_RE = re.compile(r"(\d{1,2}\.\d{2})\.?\d+\)")
SPACED_FOOTNOTE_RE = re.compile(r"(\d{1,2}\.\d{2}\.?)\s+\d+\)")
EMPTY_PLACEHOLDER_FOOTNOTE_RE = re.compile(r"(-+)\d+\)")
TRAILING_PAREN_FOOTNOTE_RE = re.compile(r"(?<=\.)\d+\)$")
TRAILING_SPACED_FOOTNOTE_RE = re.compile(r"(?<=\.)\s+\d+$")
TRAILING_APPENDED_SINGLE_DAY_FOOTNOTE_RE = re.compile(r"(\d{1,2}\.\d{2}\.?)\d$")
TRAILING_APPENDED_RANGE_FOOTNOTE_RE = re.compile(
    r"(\d{1,2}\.\d{1,2}\.?\s*-\s*\d{1,2}\.\d{2}\.?)\d$"
)
BROKEN_DAY_MONTH_SEPARATOR_RE = re.compile(r"(?<!\.\d)(?<!\.\d\d)(?<=\d)-(?=\d{1,2}\.)")
DOUBLE_DOT_RANGE_RE = re.compile(r"\.\s*\.\s*-\s*")
SHARED_MONTH_SPACED_RE = re.compile(r"(?<!\d\.)\b(\d{1,2})\.\s*/\s*(\d{1,2})\.(\d{1,2})\.")
SHARED_MONTH_COMPACT_RE = re.compile(r"(?<!\d\.)\b(\d{1,2})\./(\d{1,2})\.(\d{1,2})\.")
MULTIPLE_DOTS_RE = re.compile(r"\.{2,}")
MULTIPLE_SLASHES_RE = re.compile(r"/{2,}")
ONLY_DASHES_RE = re.compile(r"-+")


class _AnchorCollector(HTMLParser):
    def __init__(self) -> None:
        super().__init__()
        self.links: list[tuple[str, str]] = []
        self._href: str | None = None
        self._text_parts: list[str] = []

    def handle_starttag(self, tag: str, attrs: list[tuple[str, str | None]]) -> None:
        if tag != "a":
            return None
        self._href = dict(attrs).get("href")
        self._text_parts = []
        return None

    def handle_data(self, data: str) -> None:
        if self._href is not None:
            self._text_parts.append(data)

    def handle_endtag(self, tag: str) -> None:
        if tag != "a" or self._href is None:
            return None
        self.links.append((self._href, "".join(self._text_parts).strip()))
        self._href = None
        self._text_parts = []
        return None


def _require_pdfplumber():
    try:
        import pdfplumber  # type: ignore[import-not-found]
    except ImportError as exc:  # pragma: no cover - dev-only dependency.
        raise SystemExit(
            "pdfplumber is required to generate Germany school holidays data. "
            "Install it with `python -m pip install pdfplumber`."
        ) from exc
    return pdfplumber


def _normalize_short_year(value: str) -> int:
    year = int(value)
    if len(value) == 4:
        return year
    return 1900 + year if year >= 50 else 2000 + year


def _extract_start_year(label: str, href: str) -> int | None:
    if match := KMK_SCHOOL_YEAR_RE.search(label):
        return int(match.group("start"))

    filename = Path(urlparse(href).path).name
    if match := PDF_FILENAME_YEAR_RE.search(filename):
        return _normalize_short_year(match.group("start"))

    return None


def _fetch_url(url: str) -> bytes:
    with urlopen(url, timeout=URL_TIMEOUT_SECONDS) as response:  # noqa: S310
        return response.read()


def _get_school_year_pdf_links(index_url: str) -> list[tuple[int, str]]:
    parser = _AnchorCollector()
    parser.feed(_fetch_url(index_url).decode("utf-8"))

    links = {}
    for href, label in parser.links:
        full_url = urljoin(index_url, href)
        if not full_url.lower().endswith(".pdf") or "schuljahr" not in label.lower():
            continue
        if (start_year := _extract_start_year(label, full_url)) is None:
            continue
        links[start_year] = full_url

    if not links:
        raise SystemExit(f"No KMK school-year PDF links found on {index_url}.")

    return sorted(
        (start_year, url)
        for start_year, url in links.items()
        if start_year >= SUPPORTED_START_YEAR
    )


def ensure_pdf_sources(raw_pdf_dir: Path, index_url: str) -> list[Path]:
    raw_pdf_dir.mkdir(parents=True, exist_ok=True)

    indexed_pdfs = {
        Path(urlparse(pdf_url).path).name: pdf_url
        for _, pdf_url in _get_school_year_pdf_links(index_url)
    }

    existing_paths = {path.name: path for path in raw_pdf_dir.glob("*.pdf")}
    for extra_name in sorted(existing_paths.keys() - indexed_pdfs.keys()):
        existing_paths[extra_name].unlink()

    pdf_paths = []
    for filename, pdf_url in sorted(indexed_pdfs.items()):
        pdf_path = raw_pdf_dir / filename
        if not pdf_path.exists():
            pdf_path.write_bytes(_fetch_url(pdf_url))
        pdf_paths.append(pdf_path)

    return pdf_paths


def _normalize_state_label(label: str) -> str:
    label = label.replace("\n", " ").strip()
    label = re.sub(r"-\s+", "-", label)
    label = re.sub(r"^\d+\s+", "", label)
    label = re.sub(r"\s*\([^)]*\)", "", label)
    label = label.replace("²", "").replace("*", "")
    label = re.sub(r"\s+\d+$", "", label)
    return re.sub(r"\s+", " ", label).strip()


def _normalize_cell_separators(cell: str) -> str:
    """Normalize KMK cell separators before date-specific cleanup."""

    cell = cell.replace("\n", "/")
    cell = cell.replace("\u2013", "-").replace("\u2014", "-")
    cell = re.sub(r"\bund\b", "/", cell)
    return cell.replace("+", "/")


def _strip_cell_footnotes(cell: str) -> str:
    """Remove KMK footnote markers without changing the underlying dates."""

    cell = NON_DATE_FOOTNOTE_RE.sub("", cell)
    cell = DATE_DAY_MONTH_FOOTNOTE_RE.sub(r"\1", cell)
    cell = SPACED_FOOTNOTE_RE.sub(r"\1", cell)
    cell = EMPTY_PLACEHOLDER_FOOTNOTE_RE.sub(r"\1", cell)
    cell = TRAILING_PAREN_FOOTNOTE_RE.sub("", cell)
    cell = TRAILING_SPACED_FOOTNOTE_RE.sub("", cell)
    cell = TRAILING_APPENDED_SINGLE_DAY_FOOTNOTE_RE.sub(r"\1", cell)
    return TRAILING_APPENDED_RANGE_FOOTNOTE_RE.sub(r"\1", cell)


def _repair_pdf_artifacts(cell: str) -> str:
    """Repair malformed date text emitted by KMK PDFs/PDF extraction."""

    cell = BROKEN_DAY_MONTH_SEPARATOR_RE.sub(".", cell)
    cell = DOUBLE_DOT_RANGE_RE.sub(".-", cell)
    return cell


def _expand_shared_month_notation(cell: str) -> str:
    """Expand forms like `20./21.06.` into explicit month-qualified dates."""

    cell = SHARED_MONTH_SPACED_RE.sub(r"\1.\3/\2.\3", cell)
    return SHARED_MONTH_COMPACT_RE.sub(r"\1.\3/\2.\3", cell)


def _collapse_cell_formatting(cell: str) -> str:
    """Collapse duplicate delimiters and normalize empty placeholders."""

    cell = re.sub(r"\s*/\s*", "/", cell)
    cell = re.sub(r"\s*-\s*", "-", cell)
    cell = cell.replace("./", "/")
    cell = MULTIPLE_DOTS_RE.sub(".", cell)
    cell = MULTIPLE_SLASHES_RE.sub("/", cell)
    cell = re.sub(r"\s+", " ", cell).strip(" ./")
    if ONLY_DASHES_RE.fullmatch(cell):
        return "--"
    return cell


def _normalize_cell(cell: str) -> str:
    cell = _normalize_cell_separators(cell)
    cell = _strip_cell_footnotes(cell)
    cell = _repair_pdf_artifacts(cell)
    cell = re.sub(r"[¹²³⁴⁵⁶⁷⁸⁹]", "", cell)
    cell = _expand_shared_month_notation(cell)
    return _collapse_cell_formatting(cell)


def _parse_header_cell(cell: str) -> tuple[str, str]:
    parts = [part.strip() for part in cell.splitlines() if part.strip()]
    if not parts:
        raise ValueError("Header cell is empty.")
    raw_label = re.sub(r"(?:\s*\d+\)|[*¹²³⁴⁵⁶⁷⁸⁹])+$", "", parts[0]).strip()
    if raw_label != "Land" and raw_label not in CANONICAL_COLUMN_KEYS:
        raise ValueError(f"Unsupported KMK header column: {raw_label!r}.")
    label = CANONICAL_COLUMN_KEYS.get(raw_label, raw_label)
    if label == "Land":
        return label, ""
    year_label = parts[1] if len(parts) > 1 else ""
    return label, year_label


def _parse_school_year(value: str) -> tuple[int, int]:
    start_year_str, end_year_str = value.split("/")
    start_year = int(start_year_str)
    if len(end_year_str) == 4:
        end_year = int(end_year_str)
    else:
        end_year = (start_year // 100) * 100 + int(end_year_str)
        if end_year <= start_year:
            end_year += 100
    return start_year, end_year


def _parse_pdf_table(path: Path) -> tuple[int, dict[str, list[tuple[str, str, str]]]]:
    pdfplumber = _require_pdfplumber()
    with pdfplumber.open(path) as pdf:
        table = pdf.pages[0].extract_table()

    if not table:
        raise ValueError(f"Unable to extract table from {path.name}.")

    headers = [_parse_header_cell(cell or "") for cell in table[0]]
    column_specs = headers[1:]
    columns = [key for key, _ in column_specs]
    unknown_columns = sorted(set(columns) - HOLIDAY_IDS.keys())
    if unknown_columns:
        raise ValueError(f"Unsupported KMK holiday columns in {path.name}: {unknown_columns}.")

    rows = {}
    for raw_row in table[1:]:
        raw_label = raw_row[0] or ""
        state_label = _normalize_state_label(raw_label)
        subdiv = STATE_LABEL_TO_CODE.get(state_label)
        if subdiv is None:
            continue
        row_cells = raw_row[1:]
        if len(row_cells) != len(columns):
            raise ValueError(
                f"Unexpected KMK column count for {state_label!r} in {path.name}: "
                f"expected {len(columns)}, got {len(row_cells)}."
            )
        rows[subdiv] = [
            (column, year_label, _normalize_cell(cell or ""))
            for (column, year_label), cell in zip(column_specs, row_cells, strict=True)
        ]

    missing_subdivisions = sorted(EXPECTED_SUBDIVISIONS - rows.keys())
    extra_subdivisions = sorted(rows.keys() - EXPECTED_SUBDIVISIONS)
    if missing_subdivisions or extra_subdivisions:
        details = []
        if missing_subdivisions:
            details.append(f"missing subdivisions: {missing_subdivisions}")
        if extra_subdivisions:
            details.append(f"unexpected subdivisions: {extra_subdivisions}")
        raise ValueError(f"Unexpected KMK subdivision set in {path.name}: {', '.join(details)}.")

    school_year = next((year_label for _, year_label in column_specs if "/" in year_label), None)
    if school_year is None:
        raise ValueError(f"Missing school-year header in {path.name}.")
    start_year, _ = _parse_school_year(school_year)

    return start_year, rows


def load_pdf_sources(
    raw_pdf_dir: Path, index_url: str
) -> list[tuple[int, dict[str, list[tuple[str, str, str]]]]]:
    pdf_paths = ensure_pdf_sources(raw_pdf_dir, index_url)
    return sorted((_parse_pdf_table(pdf_path) for pdf_path in pdf_paths), key=lambda item: item[0])


def _parse_day_month(value: str) -> tuple[int, int]:
    value = value.rstrip(".")
    day, month = value.split(".")
    return int(month), int(day)


def parse_cell_ranges(cell: str) -> list[tuple[int, int, int, int]]:
    if not cell or set(cell) <= {"-", "_"}:
        return []

    matches = []
    for token in (part.strip() for part in cell.split("/")):
        if not token:
            continue

        if match := re.fullmatch(r"(\d{1,2}\.\d{1,2}\.?)\s*-\s*(\d{1,2}\.\d{1,2}\.?)", token):
            start, end = match.groups()
            sm, sd = _parse_day_month(start)
            em, ed = _parse_day_month(end)
            matches.append((sm, sd, em, ed))
            continue

        if match := re.fullmatch(r"(\d{1,2}\.?)\s*-\s*(\d{1,2}\.\d{1,2}\.?)", token):
            start, end = match.groups()
            em, ed = _parse_day_month(end)
            sd = int(start.rstrip("."))
            matches.append((em, sd, em, ed))
            continue

        if re.fullmatch(r"\d{1,2}\.\d{1,2}\.?", token):
            month, day = _parse_day_month(token)
            matches.append((month, day, month, day))
            continue

        for match in DATE_TOKEN_RE.finditer(token):
            month, day = _parse_day_month(match.group(0))
            matches.append((month, day, month, day))

    if not matches:
        raise ValueError(f"Unsupported KMK date cell format: {cell!r}.")
    return matches


def _resolve_years(
    year_label: str, month_day_range: tuple[int, int, int, int]
) -> tuple[date, date]:
    sm, sd, em, ed = month_day_range
    if "/" in year_label:
        start_year, end_year = _parse_school_year(year_label)
        if (em, ed) < (sm, sd):
            start_date = date(start_year, sm, sd)
            end_date = date(end_year, em, ed)
        else:
            range_year = start_year if sm >= 7 else end_year
            start_date = date(range_year, sm, sd)
            end_date = date(range_year, em, ed)
        return start_date, end_date

    year = int(year_label)
    return date(year, sm, sd), date(year, em, ed)


def normalize_ranges(
    sources: list[tuple[int, dict[str, list[tuple[str, str, str]]]]],
) -> dict[int, dict[str, list[tuple[int, int, int, int, int, int, str]]]]:
    data: defaultdict[int, defaultdict[str, list[tuple[int, int, int, int, int, int, str]]]]
    data = defaultdict(lambda: defaultdict(list))

    for _, rows in sources:
        for subdiv, cells in rows.items():
            for column_key, year_label, cell in cells:
                holiday_id = HOLIDAY_IDS[column_key]
                for month_day_range in parse_cell_ranges(cell):
                    start_date, end_date = _resolve_years(year_label, month_day_range)
                    for year in range(start_date.year, end_date.year + 1):
                        year_start = date(year, 1, 1)
                        year_end = date(year, 12, 31)
                        active_start = max(start_date, year_start)
                        active_end = min(end_date, year_end)
                        if active_start > active_end:
                            continue
                        data[year][subdiv].append(
                            (
                                start_date.year - year,
                                start_date.month,
                                start_date.day,
                                end_date.year - year,
                                end_date.month,
                                end_date.day,
                                holiday_id,
                            )
                        )

    for year_data in data.values():
        for subdiv in year_data:
            year_data[subdiv] = sorted(set(year_data[subdiv]))

    return {year: dict(sorted(year_data.items())) for year, year_data in sorted(data.items())}


def render_python_module(
    data: dict[int, dict[str, list[tuple[int, int, int, int, int, int, str]]]],
) -> str:
    lines = [
        '"""Auto-generated Germany school holidays dataset from official KMK sources."""',
        "",
        "(",
        "    AUTUMN_BREAK,",
        "    CHRISTMAS_BREAK,",
        "    WINTER_BREAK,",
        "    EASTER_SPRING_BREAK,",
        "    ASCENSION_WHIT_BREAK,",
        "    SUMMER_BREAK,",
        ") = range(6)",
        "",
        "GERMANY_SCHOOL_HOLIDAYS = {",
    ]

    for year, year_data in data.items():
        lines.append(f"    {year}: {{")
        for subdiv, ranges in year_data.items():
            lines.append(f'        "{subdiv}": (')
            for start_offset, sm, sd, end_offset, em, ed, holiday_id in ranges:
                lines.append(
                    "            "
                    f"({start_offset}, {sm}, {sd}, {end_offset}, {em}, {ed}, {holiday_id}),"
                )
            lines.append("        ),")
        lines.append("    },")
    lines.extend(
        [
            "}",
            "",
            "__all__ = ("
            '"AUTUMN_BREAK", "CHRISTMAS_BREAK", "WINTER_BREAK", '
            '"EASTER_SPRING_BREAK", "ASCENSION_WHIT_BREAK", "SUMMER_BREAK", '
            '"GERMANY_SCHOOL_HOLIDAYS")',
        ]
    )
    return "\n".join(lines) + "\n"


def _parse_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser()
    parser.add_argument("--input-dir", type=Path, default=DEFAULT_RAW_PDF_DIR)
    parser.add_argument("--download-url", default=DEFAULT_KMK_PAGE_URL)
    return parser.parse_args()


def main() -> None:
    args = _parse_args()
    data = normalize_ranges(load_pdf_sources(args.input_dir, args.download_url))
    OUTPUT_PATH.write_text(render_python_module(data), encoding="utf-8", newline="\n")


if __name__ == "__main__":
    main()



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.guatemala import Guatemala
from tests.common import CommonCountryTests


class TestGuatemala(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Guatemala)

    def test_new_years_day(self):
        self.assertHolidayName("Año Nuevo", (f"{year}-01-01" for year in self.full_range))

    def test_maundy_thursday(self):
        name = "Jueves Santo"
        self.assertHolidayName(
            name,
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
            "2024-03-28",
            "2025-04-17",
        )
        self.assertHolidayName(name, self.full_range)

    def test_good_friday(self):
        name = "Viernes Santo"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_holy_saturday(self):
        name = "Sábado Santo"
        self.assertHolidayName(
            name,
            "2020-04-11",
            "2021-04-03",
            "2022-04-16",
            "2023-04-08",
            "2024-03-30",
            "2025-04-19",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        name = "Día del Trabajo"
        self.assertNonObservedHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        self.assertHolidayName(name, "2019-04-29")

    def test_army_day(self):
        name = "Día del Ejército"
        self.assertNonObservedHolidayName(name, (f"{year}-06-30" for year in self.full_range))
        self.assertHolidayName(
            name,
            "2020-06-29",
            "2021-06-28",
            "2022-07-04",
            "2023-07-03",
            "2024-07-01",
            "2025-06-30",
        )

    def test_assumption_day(self):
        self.assertHolidayName("Día de la Asunción", (f"{year}-08-15" for year in self.full_range))

    def test_independence_day(self):
        self.assertHolidayName(
            "Día de la Independencia", (f"{year}-09-15" for year in self.full_range)
        )

    def test_revolution_day(self):
        name = "Día de la Revolución"
        self.assertNonObservedHolidayName(name, (f"{year}-10-20" for year in self.full_range))
        self.assertHolidayName(
            name,
            "2018-10-22",
            "2019-10-21",
        )

    def test_all_saints_day(self):
        self.assertHolidayName(
            "Día de Todos los Santos", (f"{year}-11-01" for year in self.full_range)
        )

    def test_christmas_day(self):
        self.assertHolidayName("Día de Navidad", (f"{year}-12-25" for year in self.full_range))

    def test_2016(self):
        # https://calendariohispanohablante.com/2016/calendario-guatemala-2016.html
        self.assertHolidayDates(
            Guatemala(years=2016),
            "2016-01-01",
            "2016-03-24",
            "2016-03-25",
            "2016-03-26",
            "2016-05-01",
            "2016-06-30",
            "2016-08-15",
            "2016-09-15",
            "2016-10-20",
            "2016-11-01",
            "2016-12-25",
        )

    def test_2017(self):
        # https://calendariohispanohablante.com/2017/calendario-guatemala-2017.html
        self.assertHolidayDates(
            Guatemala(years=2017),
            "2017-01-01",
            "2017-04-13",
            "2017-04-14",
            "2017-04-15",
            "2017-05-01",
            "2017-06-30",
            "2017-08-15",
            "2017-09-15",
            "2017-10-20",
            "2017-11-01",
            "2017-12-25",
        )

    def test_2018(self):
        # https://calendariohispanohablante.com/2018/calendario-guatemala-2018.html
        self.assertHolidayDates(
            Guatemala(years=2018),
            "2018-01-01",
            "2018-03-29",
            "2018-03-30",
            "2018-03-31",
            "2018-05-01",
            "2018-06-30",
            "2018-08-15",
            "2018-09-15",
            "2018-10-22",
            "2018-11-01",
            "2018-12-25",
        )

    def test_2019(self):
        # https://calendariohispanohablante.com/2019/calendario-guatemala-2019.html
        self.assertHolidayDates(
            Guatemala(years=2019),
            "2019-01-01",
            "2019-04-18",
            "2019-04-19",
            "2019-04-20",
            "2019-04-29",
            "2019-07-01",
            "2019-08-15",
            "2019-09-15",
            "2019-10-21",
            "2019-11-01",
            "2019-12-25",
        )

    def test_2020(self):
        # https://calendariohispanohablante.com/2020/calendario-guatemala-2020.html
        self.assertHolidayDates(
            Guatemala(years=2020),
            "2020-01-01",
            "2020-04-09",
            "2020-04-10",
            "2020-04-11",
            "2020-05-01",
            "2020-06-29",
            "2020-08-15",
            "2020-09-15",
            "2020-10-20",
            "2020-11-01",
            "2020-12-25",
        )

    def test_2021(self):
        # https://calendariohispanohablante.com/2021/calendario-guatemala-2021.html
        self.assertHolidayDates(
            Guatemala(years=2021),
            "2021-01-01",
            "2021-04-01",
            "2021-04-02",
            "2021-04-03",
            "2021-05-01",
            "2021-06-28",
            "2021-08-15",
            "2021-09-15",
            "2021-10-20",
            "2021-11-01",
            "2021-12-25",
        )

    def test_2022(self):
        # https://publicholidays.la/guatemala/es/2022-dates/
        self.assertHolidayDates(
            Guatemala(years=2022),
            "2022-01-01",
            "2022-04-14",
            "2022-04-15",
            "2022-04-16",
            "2022-05-01",
            "2022-07-04",
            "2022-08-15",
            "2022-09-15",
            "2022-10-20",
            "2022-11-01",
            "2022-12-25",
        )

    def test_2023(self):
        # https://publicholidays.la/guatemala/es/2023-dates/
        self.assertHolidayDates(
            Guatemala(years=2023),
            "2023-01-01",
            "2023-04-06",
            "2023-04-07",
            "2023-04-08",
            "2023-05-01",
            "2023-07-03",
            "2023-08-15",
            "2023-09-15",
            "2023-10-20",
            "2023-11-01",
            "2023-12-25",
        )

    def test_2024(self):
        # https://publicholidays.la/guatemala/es/2024-dates/
        self.assertHolidayDates(
            Guatemala(years=2024),
            "2024-01-01",
            "2024-03-28",
            "2024-03-29",
            "2024-03-30",
            "2024-05-01",
            "2024-07-01",
            "2024-08-15",
            "2024-09-15",
            "2024-10-20",
            "2024-11-01",
            "2024-12-25",
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Año Nuevo"),
            ("2024-03-28", "Jueves Santo"),
            ("2024-03-29", "Viernes Santo"),
            ("2024-03-30", "Sábado Santo"),
            ("2024-05-01", "Día del Trabajo"),
            ("2024-07-01", "Día del Ejército"),
            ("2024-08-15", "Día de la Asunción"),
            ("2024-09-15", "Día de la Independencia"),
            ("2024-10-20", "Día de la Revolución"),
            ("2024-11-01", "Día de Todos los Santos"),
            ("2024-12-25", "Día de Navidad"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-03-28", "Maundy Thursday"),
            ("2024-03-29", "Good Friday"),
            ("2024-03-30", "Holy Saturday"),
            ("2024-05-01", "Labor Day"),
            ("2024-07-01", "Army Day"),
            ("2024-08-15", "Assumption Day"),
            ("2024-09-15", "Independence Day"),
            ("2024-10-20", "Revolution Day"),
            ("2024-11-01", "All Saints' Day"),
            ("2024-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.jersey import Jersey
from tests.common import CommonCountryTests


class TestJersey(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Jersey)

    def test_special_holidays(self):
        dt = (
            "1957-07-26",
            "1977-06-07",
            "1978-06-27",
            "1981-07-29",
            "1989-05-25",
            "1999-12-31",
            "2001-07-13",
            "2002-06-03",
            "2011-04-29",
            "2012-06-05",
            "2020-05-04",
            "2020-05-08",
            "2021-09-27",
            "2022-06-03",
            "2022-09-19",
            "2023-05-08",
            "2024-07-15",
        )
        dt_observed = (
            "1976-12-28",
            "1977-01-03",
            "1981-12-28",
            "1982-12-28",
            "1983-01-03",
            "1987-12-28",
            "1992-12-28",
            "1993-12-28",
            "1994-01-03",
            "1998-12-28",
            "1999-12-28",
            "2000-01-03",
            "2009-12-28",
        )
        self.assertHoliday(dt, dt_observed)
        self.assertNoNonObservedHoliday(dt_observed)

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2011-01-03",
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (substitute day)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_early_may_bank_holiday(self):
        name = "Early May Bank Holiday"
        name_2024 = "May Bank Holiday"
        self.assertHolidayName(
            name,
            "1995-05-08",
            "2020-05-04",
            "2021-05-03",
            "2022-05-02",
            "2023-05-01",
            "2025-05-05",
        )
        self.assertHolidayName(name, range(1980, 2024), range(2025, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1980), 2024)

        self.assertHolidayName(name_2024, "2024-05-06")
        self.assertNoHolidayName(
            name_2024, range(self.start_year, 2024), range(2025, self.end_year)
        )

    def test_liberation_day(self):
        name = "Liberation Day"
        self.assertNonObservedHolidayName(name, (f"{year}-05-09" for year in self.full_range))
        self.assertNoHolidayName(
            name,
            # Sunday, May 9th list 1952-2050 excl. 2010 (65th Anniversary)
            "1954-05-09",
            "1965-05-09",  # 20th Anniversary special? need to check
            "1971-05-09",
            "1976-05-09",
            "1982-05-09",
            "1993-05-09",
            "1999-05-09",
            "2004-05-09",
            "2021-05-09",
            "2027-05-09",
            "2032-05-09",
            "2038-05-09",
            "2049-05-09",
            # Saturday, May 9th list 1952-2010
            "1953-05-09",
            "1959-05-09",
            "1964-05-09",
            "1970-05-09",  # 25th Anniversary special? need to check
            "1981-05-09",
            "1987-05-09",
            "1992-05-09",
            "1998-05-09",
            "2009-05-09",
        )

    def test_spring_bank_holiday(self):
        name = "Spring Bank Holiday"
        self.assertHolidayName(
            name,
            "2002-06-04",
            "2012-06-04",
            "2020-05-25",
            "2021-05-31",
            "2022-06-02",
            "2023-05-29",
            "2024-05-27",
            "2025-05-26",
        )
        self.assertHolidayName(name, range(1970, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1970))

    def test_whit_monday(self):
        name = "Whit Monday"
        self.assertHolidayName(
            name,
            "1964-05-18",
            "1965-06-07",
            "1966-05-30",
            "1967-05-15",
            "1968-06-03",
            "1969-05-26",
        )
        self.assertHolidayName(name, range(self.start_year, 1970))
        self.assertNoHolidayName(name, range(1970, self.end_year))

    def test_summer_bank_holiday(self):
        name = "Summer Bank Holiday"
        self.assertHolidayName(
            name,
            "1964-09-07",
            "1965-09-06",
            "1966-09-05",
            "1967-09-04",
            "1968-09-02",
            "1969-09-01",
            "2020-08-31",
            "2021-08-30",
            "2022-08-29",
            "2023-08-28",
            "2024-08-26",
            "2025-08-25",
        )
        self.assertHolidayName(name, self.full_range)

    def test_2010(self):
        # Wayback Machine of https://www.gov.je/Leisure/Events/WhatsOn/Pages/BankHolidayDates.aspx
        self.assertHolidaysInYear(
            2010,
            ("2010-01-01", "New Year's Day"),
            ("2010-04-02", "Good Friday"),
            ("2010-04-05", "Easter Monday"),
            ("2010-05-03", "Early May Bank Holiday"),
            ("2010-05-09", "Liberation Day"),
            ("2010-05-31", "Spring Bank Holiday"),
            ("2010-08-30", "Summer Bank Holiday"),
            ("2010-12-25", "Christmas Day"),
            ("2010-12-26", "Boxing Day"),
            ("2010-12-27", "Christmas Day (substitute day)"),
            ("2010-12-28", "Boxing Day (substitute day)"),
        )

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2010-12-27",
            "2011-12-27",
            "2016-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (substitute day)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2009-12-28",
            "2010-12-28",
            "2015-12-28",
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (substitute day)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2011(self):
        # Wayback Machine of https://www.gov.je/Leisure/Events/WhatsOn/Pages/BankHolidayDates.aspx
        self.assertHolidaysInYear(
            2011,
            ("2011-01-01", "New Year's Day"),
            ("2011-01-03", "New Year's Day (substitute day)"),
            ("2011-04-22", "Good Friday"),
            ("2011-04-25", "Easter Monday"),
            ("2011-04-29", "Wedding of William and Catherine"),
            ("2011-05-02", "Early May Bank Holiday"),
            ("2011-05-09", "Liberation Day"),
            ("2011-05-30", "Spring Bank Holiday"),
            ("2011-08-29", "Summer Bank Holiday"),
            ("2011-12-25", "Christmas Day"),
            ("2011-12-26", "Boxing Day"),
            ("2011-12-27", "Christmas Day (substitute day)"),
        )

    def test_2012(self):
        # Wayback Machine of https://www.gov.je/Leisure/Events/WhatsOn/Pages/BankHolidayDates.aspx
        self.assertHolidaysInYear(
            2012,
            ("2012-01-01", "New Year's Day"),
            ("2012-01-02", "New Year's Day (substitute day)"),
            ("2012-04-06", "Good Friday"),
            ("2012-04-09", "Easter Monday"),
            ("2012-05-07", "Early May Bank Holiday"),
            ("2012-05-09", "Liberation Day"),
            ("2012-06-04", "Spring Bank Holiday"),
            ("2012-06-05", "Queen's Diamond Jubilee"),
            ("2012-08-27", "Summer Bank Holiday"),
            ("2012-12-25", "Christmas Day"),
            ("2012-12-26", "Boxing Day"),
        )

    def test_2013(self):
        # Wayback Machine of https://www.gov.je/Leisure/Events/WhatsOn/Pages/BankHolidayDates.aspx
        self.assertHolidaysInYear(
            2013,
            ("2013-01-01", "New Year's Day"),
            ("2013-03-29", "Good Friday"),
            ("2013-04-01", "Easter Monday"),
            ("2013-05-06", "Early May Bank Holiday"),
            ("2013-05-09", "Liberation Day"),
            ("2013-05-27", "Spring Bank Holiday"),
            ("2013-08-26", "Summer Bank Holiday"),
            ("2013-12-25", "Christmas Day"),
            ("2013-12-26", "Boxing Day"),
        )

    def test_2014(self):
        # Wayback Machine of https://www.gov.je/Leisure/Events/WhatsOn/Pages/BankHolidayDates.aspx
        self.assertHolidaysInYear(
            2014,
            ("2014-01-01", "New Year's Day"),
            ("2014-04-18", "Good Friday"),
            ("2014-04-21", "Easter Monday"),
            ("2014-05-05", "Early May Bank Holiday"),
            ("2014-05-09", "Liberation Day"),
            ("2014-05-26", "Spring Bank Holiday"),
            ("2014-08-25", "Summer Bank Holiday"),
            ("2014-12-25", "Christmas Day"),
            ("2014-12-26", "Boxing Day"),
        )

    def test_2015(self):
        # Wayback Machine of https://www.gov.je/Leisure/Events/WhatsOn/Pages/BankHolidayDates.aspx
        self.assertHolidaysInYear(
            2015,
            ("2015-01-01", "New Year's Day"),
            ("2015-04-03", "Good Friday"),
            ("2015-04-06", "Easter Monday"),
            ("2015-05-04", "Early May Bank Holiday"),
            ("2015-05-09", "Liberation Day"),
            ("2015-05-25", "Spring Bank Holiday"),
            ("2015-08-31", "Summer Bank Holiday"),
            ("2015-12-25", "Christmas Day"),
            ("2015-12-26", "Boxing Day"),
            ("2015-12-28", "Boxing Day (substitute day)"),
        )

    def test_2016(self):
        # Wayback Machine of https://www.gov.je/Leisure/Events/WhatsOn/Pages/BankHolidayDates.aspx
        self.assertHolidaysInYear(
            2016,
            ("2016-01-01", "New Year's Day"),
            ("2016-03-25", "Good Friday"),
            ("2016-03-28", "Easter Monday"),
            ("2016-05-02", "Early May Bank Holiday"),
            ("2016-05-09", "Liberation Day"),
            ("2016-05-30", "Spring Bank Holiday"),
            ("2016-08-29", "Summer Bank Holiday"),
            ("2016-12-25", "Christmas Day"),
            ("2016-12-26", "Boxing Day"),
            ("2016-12-27", "Christmas Day (substitute day)"),
        )

    def test_2017(self):
        # Wayback Machine of https://www.gov.je/Leisure/Events/WhatsOn/Pages/BankHolidayDates.aspx
        self.assertHolidaysInYear(
            2017,
            ("2017-01-01", "New Year's Day"),
            ("2017-01-02", "New Year's Day (substitute day)"),
            ("2017-04-14", "Good Friday"),
            ("2017-04-17", "Easter Monday"),
            ("2017-05-01", "Early May Bank Holiday"),
            ("2017-05-09", "Liberation Day"),
            ("2017-05-29", "Spring Bank Holiday"),
            ("2017-08-28", "Summer Bank Holiday"),
            ("2017-12-25", "Christmas Day"),
            ("2017-12-26", "Boxing Day"),
        )

    def test_2018(self):
        # Wayback Machine of https://www.gov.je/Leisure/Events/WhatsOn/Pages/BankHolidayDates.aspx
        self.assertHolidaysInYear(
            2018,
            ("2018-01-01", "New Year's Day"),
            ("2018-03-30", "Good Friday"),
            ("2018-04-02", "Easter Monday"),
            ("2018-05-07", "Early May Bank Holiday"),
            ("2018-05-09", "Liberation Day"),
            ("2018-05-28", "Spring Bank Holiday"),
            ("2018-08-27", "Summer Bank Holiday"),
            ("2018-12-25", "Christmas Day"),
            ("2018-12-26", "Boxing Day"),
        )

    def test_2019(self):
        # Wayback Machine of https://www.gov.je/Leisure/Events/WhatsOn/Pages/BankHolidayDates.aspx
        self.assertHolidaysInYear(
            2019,
            ("2019-01-01", "New Year's Day"),
            ("2019-04-19", "Good Friday"),
            ("2019-04-22", "Easter Monday"),
            ("2019-05-06", "Early May Bank Holiday"),
            ("2019-05-09", "Liberation Day"),
            ("2019-05-27", "Spring Bank Holiday"),
            ("2019-08-26", "Summer Bank Holiday"),
            ("2019-12-25", "Christmas Day"),
            ("2019-12-26", "Boxing Day"),
        )

    def test_2020(self):
        # Wayback Machine of https://www.gov.je/Leisure/Events/WhatsOn/Pages/BankHolidayDates.aspx
        self.assertHolidaysInYear(
            2020,
            ("2020-01-01", "New Year's Day"),
            ("2020-04-10", "Good Friday"),
            ("2020-04-13", "Easter Monday"),
            ("2020-05-04", "Early May Bank Holiday"),
            ("2020-05-08", "75th Anniversary of VE Day"),
            ("2020-05-09", "Liberation Day"),
            ("2020-05-25", "Spring Bank Holiday"),
            ("2020-08-31", "Summer Bank Holiday"),
            ("2020-12-25", "Christmas Day"),
            ("2020-12-26", "Boxing Day"),
            ("2020-12-28", "Boxing Day (substitute day)"),
        )

    def test_2021(self):
        # Wayback Machine of https://www.gov.je/Leisure/Events/WhatsOn/Pages/BankHolidayDates.aspx
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "New Year's Day"),
            ("2021-04-02", "Good Friday"),
            ("2021-04-05", "Easter Monday"),
            ("2021-05-03", "Early May Bank Holiday"),
            ("2021-05-31", "Spring Bank Holiday"),
            ("2021-08-30", "Summer Bank Holiday"),
            ("2021-09-27", "Corn Riots Anniversary"),
            ("2021-12-25", "Christmas Day"),
            ("2021-12-26", "Boxing Day"),
            ("2021-12-27", "Christmas Day (substitute day)"),
            ("2021-12-28", "Boxing Day (substitute day)"),
        )

    def test_2022(self):
        # Wayback Machine of https://www.gov.je/Leisure/Events/WhatsOn/Pages/BankHolidayDates.aspx
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (substitute day)"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-02", "Early May Bank Holiday"),
            ("2022-05-09", "Liberation Day"),
            ("2022-06-02", "Spring Bank Holiday"),
            ("2022-06-03", "Queen's Platinum Jubilee"),
            ("2022-08-29", "Summer Bank Holiday"),
            ("2022-09-19", "Funeral of Her Majesty Queen Elizabeth II"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (substitute day)"),
        )

    def test_2023(self):
        # https://www.gov.je/Leisure/Events/WhatsOn/Pages/BankHolidayDates.aspx
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Day (substitute day)"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-10", "Easter Monday"),
            ("2023-05-01", "Early May Bank Holiday"),
            ("2023-05-08", "Coronation of His Majesty King Charles III"),
            ("2023-05-09", "Liberation Day"),
            ("2023-05-29", "Spring Bank Holiday"),
            ("2023-08-28", "Summer Bank Holiday"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Boxing Day"),
        )

    def test_2024(self):
        # https://www.gov.je/Leisure/Events/WhatsOn/Pages/BankHolidayDates.aspx
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-06", "May Bank Holiday"),
            ("2024-05-09", "Liberation Day"),
            ("2024-05-27", "Spring Bank Holiday"),
            ("2024-07-15", "The visit of His Majesty King Charles III and Queen Camilla"),
            ("2024-08-26", "Summer Bank Holiday"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )

    def test_2025(self):
        # https://www.gov.je/Leisure/Events/WhatsOn/Pages/BankHolidayDates.aspx
        self.assertHolidaysInYear(
            2025,
            ("2025-01-01", "New Year's Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-05", "Early May Bank Holiday"),
            ("2025-05-09", "Liberation Day"),
            ("2025-05-26", "Spring Bank Holiday"),
            ("2025-08-25", "Summer Bank Holiday"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.calendars.gregorian import FEB, MAR, SEP, DEC
from holidays.constants import OPTIONAL, PUBLIC
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.holiday_base import HolidayBase


class Ireland(HolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Ireland holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_the_Republic_of_Ireland>
        * [Public Holidays in Ireland](https://web.archive.org/web/20250317124917/https://www.citizensinformation.ie/en/employment/employment_rights_and_conditions/leave_and_holidays/public_holidays_in_ireland.html)
    """

    country = "IE"
    supported_categories = (OPTIONAL, PUBLIC)
    start_year = 1872

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, IrelandStaticHolidays)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        if self._year >= 1975:
            self._add_new_years_day("New Year's Day")

        # Saint Brigid's Day.
        if self._year >= 2023:
            name = "Saint Brigid's Day"
            if self._is_friday(FEB, 1):
                self._add_holiday_feb_1(name)
            else:
                self._add_holiday_1st_mon_from_feb_1(name)

        # Saint Patrick's Day.
        if self._year >= 1903:
            self._add_saint_patricks_day("Saint Patrick's Day")

        # Easter Monday.
        self._add_easter_monday("Easter Monday")

        # May Day.
        if self._year >= 1994:
            name = "May Day"
            if self._year == 1995:
                self._add_holiday_may_8(name)
            else:
                self._add_holiday_1st_mon_of_may(name)

        if self._year >= 1973:
            # June Bank Holiday.
            self._add_holiday_1st_mon_of_jun("June Bank Holiday")
        else:
            # Whit Monday.
            self._add_whit_monday("Whit Monday")

        # August Bank Holiday.
        self._add_holiday_1st_mon_of_aug("August Bank Holiday")

        # October Bank Holiday.
        if self._year >= 1977:
            self._add_holiday_last_mon_of_oct("October Bank Holiday")

        # Christmas Day.
        self._add_christmas_day("Christmas Day")

        # Saint Stephen's Day.
        self._add_christmas_day_two("Saint Stephen's Day")

    def _populate_optional_holidays(self):
        # Good Friday.
        self._add_good_friday("Good Friday")


class IE(Ireland):
    pass


class IRL(Ireland):
    pass


class IrelandStaticHolidays:
    special_public_holidays = {
        1999: (DEC, 31, "Millennium Celebrations"),
        2011: (SEP, 14, "National Day of Mourning"),
        2022: (MAR, 18, "Day of Remembrance and Recognition"),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.groups import ChristianHolidays
from holidays.holiday_base import HolidayBase


class TestChristianHolidays(TestCase):
    def test_check_calendar(self):
        self.assertRaises(ValueError, lambda: ChristianHolidays("INVALID_CALENDAR"))

    def test_add_christmas_day_three(self):
        class TestHolidays(HolidayBase, ChristianHolidays):
            def __init__(self, *args, **kwargs):
                ChristianHolidays.__init__(self)
                super().__init__(*args, **kwargs)

            def _populate(self, year):
                super()._populate(year)

        test_holidays = TestHolidays()

        test_holidays._populate(2022)
        test_holidays._add_christmas_day_three("Third day")
        self.assertIn("2022-12-27", test_holidays)
        self.assertEqual(1, len(test_holidays))
